@@ -252,8 +252,7 @@ impl_Galeon_Browser_loadurl(impl_POA_Galeon_Browser * servant,
 		case Galeon_TAB_FORCE:
 		case Galeon_TAB_DEFAULT:
 			embed_create_after_embed (prev_embed, FALSE, url,
-						  EMBED_CREATE_FORCE_JUMP |
-						  EMBED_CREATE_RAISE_WINDOW);
+						  EMBED_CREATE_FORCE_JUMP);
 			break;
 
 		/* open in tab, and don't raise the window */
