@@ -17,15 +17,18 @@
  */
 
 #include "galeon.h"
-#include "bookmarks.h"
+#include "bookmarks_editor.h"
 #include "bookmarks_editor_callbacks.h"
-#include "misc.h"
+#include "misc_string.h"
+#include "misc_gui.h"
 #include "glade.h"
 #include "window.h"
 #include "state.h"
 #include "prefs.h"
 #include "context.h"
 #include "dialog.h"
+#include "favicon.h"
+#include "toolbar.h"
 
 #include <string.h>
 #include <gtk/gtktoolbar.h>
@@ -51,6 +54,9 @@ gboolean block_search_position = FALSE;
 /* block popping up the dock menu */
 static gboolean block_dock_popup_menu = FALSE;
 
+/* block the dock menuitem */
+gboolean block_docking = FALSE;
+
 /* Function prototypes */
 static BookmarksEditorControls *bookmarks_editor_controls_init
                                (BookmarkItem *root,
@@ -62,9 +68,7 @@ static void bookmarks_editor_add_menubar (BookmarksEditorControls *controls);
 static void
 bookmarks_editor_fill_menu (GtkMenuShell *menushell, 
 			    GtkAccelGroup *accel, 
-			    BookmarksEditorControls *controls,
-			    GtkWidget **edit_menuitem,
-			    GtkWidget **set_default_folder_menuitem);
+			    BookmarksEditorControls *controls);
 
 
 static BookmarksEditorControls *
@@ -72,7 +76,7 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 				BookmarksEditorControlsType type)
 {
 	BookmarksEditorControls *editor;
-	GtkWidget *close, *detach, *show_menu, *dummy;
+	GtkWidget *close, *detach, *show_menu, *dummy, *remove_conf;
 	GladeXML *gxml = NULL;
 	GList *toolbar_items, *tlist;
 	GtkStyle *style, *togglestyle;
@@ -105,6 +109,8 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 			(gxml, "bookmarks_editor_name_entry");
 		editor->url_entry = glade_xml_get_widget
 			(gxml, "bookmarks_editor_url_entry");
+		editor->smarturl_entry = glade_xml_get_widget
+			(gxml, "bookmarks_editor_smart_url_entry");
 		editor->nick_entry = glade_xml_get_widget
 			(gxml, "bookmarks_editor_nick_entry");
 		editor->pixmap_entry = glade_xml_get_widget
@@ -123,12 +129,15 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 			(gxml, "bookmarks_editor_time_modified_entry");
 		editor->time_visited_entry = glade_xml_get_widget
 			(gxml, "bookmarks_editor_time_visited_entry");
+		editor->edit_button = glade_xml_get_widget
+			(gxml, "bookmarks_editor_edit_button");
 		editor->search_entry = NULL;
 		bookmarks_editor_add_menubar (editor);
 		g_assert (editor->dialog != NULL);
 		g_assert (editor->toolbar != NULL);
 		g_assert (editor->name_entry != NULL);
 		g_assert (editor->url_entry != NULL);
+		g_assert (editor->smarturl_entry != NULL);
 		g_assert (editor->nick_entry != NULL);
 		g_assert (editor->pixmap_entry != NULL);
 		g_assert (editor->pixmap_file_entry != NULL);
@@ -143,6 +152,7 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 	case DOCKED_BOOKMARKS_EDITOR:
 		gxml = glade_widget_new ("bookmarks.glade", "bookmarks_dock", 
 					 &(editor->dialog), editor);
+		editor->menubar = NULL;
 		editor->toolbar = glade_xml_get_widget 
 			(gxml, "bookmarks_dock_toolbar");
 		editor->ctree = glade_xml_get_widget
@@ -154,12 +164,16 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 			(gxml, "bookmarks_dock_name_entry");
 		editor->url_entry = glade_xml_get_widget
 			(gxml, "bookmarks_dock_url_entry");
+		editor->smarturl_entry = glade_xml_get_widget
+			(gxml, "bookmarks_dock_smart_url_entry");
 		editor->nick_entry = glade_xml_get_widget
 			(gxml, "bookmarks_dock_nick_entry");
 		editor->notes_text = glade_xml_get_widget
 			(gxml, "bookmarks_dock_notes_text");
 		editor->search_entry = glade_xml_get_widget
 			(gxml, "bookmarks_dock_search_entry");
+		editor->edit_button = glade_xml_get_widget
+			(gxml, "bookmarks_dock_edit_button");
 		editor->pixmap_entry = NULL;
 		editor->pixmap_file_entry = NULL;
 		editor->create_toolbar_toggle = NULL;
@@ -175,6 +189,7 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 		g_assert (editor->toolbar != NULL);
 		g_assert (editor->name_entry != NULL);
 		g_assert (editor->url_entry != NULL);
+		g_assert (editor->smarturl_entry != NULL);
 		g_assert (editor->nick_entry != NULL);
 		g_assert (editor->notes_text != NULL);
 		g_assert (editor->search_entry != NULL);
@@ -191,6 +206,70 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 		gtk_button_set_relief (GTK_BUTTON (show_menu),
 				       GTK_RELIEF_NONE);
 		break;
+	case REMOVE_BOOKMARKS_EDITOR:
+		gxml = glade_widget_new ("bookmarks.glade", 
+					 "remove_bookmarks_editor",
+					 &(editor->dialog), editor);
+		editor->menubar = NULL;
+		editor->toolbar = NULL;
+		editor->menubar_dock = NULL;
+		editor->toolbar_dock = NULL;
+		editor->ctree = glade_xml_get_widget
+			(gxml, "remove_bookmarks_ctree");
+		editor->vpane = NULL;
+		editor->edit_frame = glade_xml_get_widget
+			(gxml, "remove_bookmarks_edit_frame");
+		editor->name_entry = glade_xml_get_widget
+			(gxml, "remove_bookmarks_name_entry");
+		editor->url_entry = glade_xml_get_widget
+			(gxml, "remove_bookmarks_url_entry");
+		editor->smarturl_entry = glade_xml_get_widget
+			(gxml, "remove_bookmarks_smart_url_entry");
+		editor->nick_entry = glade_xml_get_widget
+			(gxml, "remove_bookmarks_nick_entry");
+		editor->pixmap_entry = glade_xml_get_widget
+			(gxml, "remove_bookmarks_pixmap_entry");
+		editor->pixmap_file_entry = glade_xml_get_widget
+			(gxml, "remove_bookmarks_pixmap_file_entry");
+		editor->create_toolbar_toggle = glade_xml_get_widget
+			(gxml, "remove_bookmarks_create_toolbar_toggle");
+		editor->create_context_menu_toggle = glade_xml_get_widget
+			(gxml, "remove_bookmarks_create_context_menu_toggle");
+		editor->notes_text = glade_xml_get_widget
+			(gxml, "remove_bookmarks_notes_text");
+		editor->time_added_entry = glade_xml_get_widget
+			(gxml, "remove_bookmarks_time_added_entry");
+		editor->time_modified_entry = glade_xml_get_widget
+			(gxml, "remove_bookmarks_time_modified_entry");
+		editor->time_visited_entry = glade_xml_get_widget
+			(gxml, "remove_bookmarks_time_visited_entry");
+		editor->edit_button = NULL;
+		editor->search_entry = NULL;
+		g_assert (editor->dialog != NULL);
+		g_assert (editor->name_entry != NULL);
+		g_assert (editor->url_entry != NULL);
+		g_assert (editor->smarturl_entry != NULL);
+		g_assert (editor->nick_entry != NULL);
+		g_assert (editor->pixmap_entry != NULL);
+		g_assert (editor->pixmap_file_entry != NULL);
+		g_assert (editor->create_toolbar_toggle != NULL);
+		g_assert (editor->create_context_menu_toggle != NULL);
+		g_assert (editor->notes_text != NULL);
+		g_assert (editor->time_added_entry != NULL);
+		g_assert (editor->time_modified_entry != NULL);
+		g_assert (editor->time_visited_entry != NULL);
+		remove_conf = glade_xml_get_widget 
+			(gxml, "remove_confirmation_check");
+		g_assert (GTK_IS_TOGGLE_BUTTON (remove_conf));
+		/* NOTE: the following will never be true actually, because 
+		   the checkbox is precisely for disabling this window...
+		   It's here just for completness */
+		gtk_toggle_button_set_active 
+			(GTK_TOGGLE_BUTTON (remove_conf),
+			 eel_gconf_get_integer 
+			 (CONF_BOOKMARKS_REMOVE_CONFIRMATION_MIN) == -1);
+		break;
+
 	default:
 		g_error ("Incorrect bookmarks editor dialog type");
 	}
@@ -202,10 +281,6 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 	editor->window = NULL;
 	editor->find_dialog = NULL;
 
-	/* get the widgets we need to work with later*/
-	editor->edit_button = glade_xml_get_widget
-			      (gxml, "bookmarks_editor_edit_button");
-
 	/* unref the GladeXML object, we don't use it anymore */
 	gtk_object_unref (GTK_OBJECT (gxml));
 
@@ -244,69 +319,75 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 	}
 
 	/* setup the context menu callback and the styles for the toolbar */
-	toolbar_items = GTK_TOOLBAR (editor->toolbar)->children; 
-	dummy = gtk_button_new ();
-	style = gtk_rc_get_style (dummy);
-	gtk_widget_destroy (dummy);
-
-	if (style == NULL)
+	if (editor->toolbar)
 	{
-		style = gtk_style_new ();
-	}
-	else
-	{
-		gtk_style_ref (style);
-	}
-
-	dummy = gtk_toggle_button_new ();
-	togglestyle = gtk_rc_get_style (dummy);
-	gtk_widget_destroy (dummy);
-
-	if (togglestyle == NULL)
-	{
-		togglestyle = gtk_style_new ();
-	}
-	else
-	{
-		gtk_style_ref (togglestyle);
-	}
-
-	for (tlist = toolbar_items; tlist; tlist = g_list_next (tlist))
-	{
-		if (!GTK_IS_WIDGET (((GtkToolbarChild*)(tlist->data))->widget))
-			continue;
-
-		gtk_signal_connect 
-			(GTK_OBJECT (((GtkToolbarChild*)(tlist->data))->widget),
-			"button-press-event",
-			GTK_SIGNAL_FUNC 
-			 (bookmarks_editor_toolbar_button_press_event_cb),
-			editor);
-
-		if (!GTK_IS_TOGGLE_BUTTON
-			(((GtkToolbarChild*)(tlist->data))->widget))
+		toolbar_items = GTK_TOOLBAR (editor->toolbar)->children; 
+		dummy = gtk_button_new ();
+		style = gtk_rc_get_style (dummy);
+		gtk_widget_destroy (dummy);
+		
+		if (style == NULL)
 		{
-			gtk_widget_set_style
-				(((GtkToolbarChild*)(tlist->data))->label,
-				 style);
+			style = gtk_style_new ();
 		}
-		else if (GTK_IS_TOGGLE_BUTTON
-			(((GtkToolbarChild*)(tlist->data))->widget))
+		else
 		{
-			gtk_widget_set_style
-				(((GtkToolbarChild*)(tlist->data))->label,
-				 togglestyle);
+			gtk_style_ref (style);
 		}
+		
+		dummy = gtk_toggle_button_new ();
+		togglestyle = gtk_rc_get_style (dummy);
+		gtk_widget_destroy (dummy);
+		
+		if (togglestyle == NULL)
+		{
+			togglestyle = gtk_style_new ();
+		}
+		else
+		{
+			gtk_style_ref (togglestyle);
+		}
+		
+		for (tlist = toolbar_items; tlist; tlist = g_list_next (tlist))
+		{
+			if (!GTK_IS_WIDGET (((GtkToolbarChild*)
+					     (tlist->data))->widget))
+				continue;
+			
+			gtk_signal_connect 
+				(GTK_OBJECT (((GtkToolbarChild*)
+					      (tlist->data))->widget),
+				 "button-press-event",
+				 GTK_SIGNAL_FUNC 
+				 (bookmarks_editor_toolbar_button_press_event_cb),
+				 editor);
+			
+			if (!GTK_IS_TOGGLE_BUTTON
+			    (((GtkToolbarChild*)(tlist->data))->widget))
+			{
+				gtk_widget_set_style
+					(((GtkToolbarChild*)
+					  (tlist->data))->label,
+					 style);
+			}
+			else if (GTK_IS_TOGGLE_BUTTON
+				 (((GtkToolbarChild*)(tlist->data))->widget))
+			{
+				gtk_widget_set_style
+					(((GtkToolbarChild*)
+					  (tlist->data))->label,
+					 togglestyle);
+			}
+		}
+		
+		gtk_style_unref (style);
+		gtk_style_unref (togglestyle);
 	}
-
-	gtk_style_unref (style);
-	gtk_style_unref (togglestyle);
-
 	/* callback to make sure that everything is cleaned before destroying 
 	   the widgets */
 	gtk_signal_connect (GTK_OBJECT (editor->ctree), "destroy",
 			    bookmarks_editor_destroy_cb, editor);
-
+	
 	bookmarks_editor_place_tree_item (editor, root);
 	return editor;
 }
@@ -314,20 +395,8 @@ bookmarks_editor_controls_init (BookmarkItem *root,
 BookmarksEditorControls *
 bookmarks_editor_show_dialog (GaleonWindow *window)
 {
-	gboolean toolbar_relief;
-	gboolean menubar_relief;
-	gboolean toolbar_detachable;
-	gboolean menubar_detachable;
-	gboolean toolbar_lines;
 	BookmarksEditorControls *controls = NULL;
 
-	/* read preferences */
-	toolbar_detachable = gnome_preferences_get_toolbar_detachable ();
-	menubar_detachable = gnome_preferences_get_menubar_detachable ();
-	toolbar_relief = gnome_preferences_get_toolbar_relief ();
-	menubar_relief = gnome_preferences_get_menubar_relief ();
-	toolbar_lines = gnome_preferences_get_toolbar_lines ();
-
 	controls = bookmarks_editor_controls_init
 		(bookmarks_root, STANDARD_BOOKMARKS_EDITOR);
 	bookmarks_editors = g_list_prepend (bookmarks_editors, controls);
@@ -342,45 +411,39 @@ bookmarks_editor_show_dialog (GaleonWindow *window)
 	
 	g_return_val_if_fail (controls != NULL, NULL);
 	
-	/* setup toolbar*/
-	gtk_toolbar_set_style 
-		(GTK_TOOLBAR (controls->toolbar),
+	/* setup toolbars */
+	toolbar_set_dock_style (controls->toolbar_dock, TOOLBAR_TYPE_TOOLBAR,
 		eel_gconf_get_integer
 			(CONF_TOOLBAR_BOOKMARKS_EDITOR_STYLE));
-	gnome_dock_item_set_shadow_type
-		(GNOME_DOCK_ITEM (controls->toolbar_dock), 
-		 toolbar_relief ? GTK_SHADOW_OUT : GTK_SHADOW_NONE);
-	(GNOME_DOCK_ITEM (controls->toolbar_dock))->behavior &=
-		~(toolbar_detachable ? GNOME_DOCK_ITEM_BEH_LOCKED : 0);
-	if (toolbar_relief)
-	{
-		gtk_container_set_border_width 
-			(GTK_CONTAINER (controls->toolbar_dock), 2);
-	}
+	toolbar_set_dock_style (controls->menubar_dock, TOOLBAR_TYPE_MENUBAR,
+				0);
 	
-	(GNOME_DOCK_ITEM (controls->menubar_dock))->behavior &=
-		~(menubar_detachable ? GNOME_DOCK_ITEM_BEH_LOCKED : 0);
-	gnome_dock_item_set_shadow_type
-		(GNOME_DOCK_ITEM (controls->menubar_dock),
-		 menubar_relief ? GTK_SHADOW_OUT : GTK_SHADOW_NONE);
+	/* get the controls visibility configuration */
+	gtk_check_menu_item_set_active 
+			(GTK_CHECK_MENU_ITEM (controls->edit_menuitem),
+			 eel_gconf_get_boolean (CONF_STATE_BM_DLG_CONTROLS));
 
-	gtk_toolbar_set_space_style (GTK_TOOLBAR (controls->toolbar),
-				     toolbar_lines ? GTK_TOOLBAR_SPACE_LINE :
-						     GTK_TOOLBAR_SPACE_EMPTY);
-		 
 	gtk_widget_show (controls->dialog);
 
 	if (window != NULL)
 	{
 		controls->window = window;
-		dialog_set_parent (controls->dialog, window->WMain);
+		dialog_set_parent (controls->dialog, window->wmain);
 	}
+
+	/* set the dock menuitem */
+	block_docking = TRUE;
+	gtk_check_menu_item_set_active
+		(GTK_CHECK_MENU_ITEM (controls->dock_menuitem), FALSE);
+	block_docking = FALSE;
+
 	return controls;
 }
 
 void
 bookmarks_editor_hide_dialog (BookmarksEditorControls *controls)
 {
+	gboolean edit_controls;
 	g_return_if_fail (controls != NULL);
 
 	/* the destroy callback of the ctree calls this function.
@@ -393,6 +456,7 @@ bookmarks_editor_hide_dialog (BookmarksEditorControls *controls)
 	 * in memory, only way out is to use revert */
 	bookmarks_updated ();
 
+	edit_controls = GTK_WIDGET_VISIBLE(controls->edit_frame);
 	if (controls->type != DOCKED_BOOKMARKS_EDITOR) 
 	{
 		// FIXME
@@ -409,13 +473,12 @@ bookmarks_editor_hide_dialog (BookmarksEditorControls *controls)
 			state_save_pane_pos (controls->vpane,
 					     "bookmarks_editor_vpane");
 		}
-		else if (controls->root_bookmark == default_bookmarks_root)
-		{
-			state_save_window_state (controls->dialog->window,
-						 "compact_bookmarks_editor");
-			state_save_dock_layout (controls->dialog,
-						 "compact_bookmarks_editor");
-		}
+		
+		eel_gconf_set_boolean (CONF_STATE_BM_DLG_CONTROLS, edit_controls);
+	}
+	else
+	{
+		eel_gconf_set_boolean (CONF_STATE_BM_DOCK_CONTROLS, edit_controls);
 	}
 	
 	gtk_clist_freeze (GTK_CLIST(controls->ctree));
@@ -469,6 +532,19 @@ bookmarks_editor_show_dock (GaleonWindow *window)
 				     toolbar_lines ? GTK_TOOLBAR_SPACE_LINE :
 						     GTK_TOOLBAR_SPACE_EMPTY);
 
+	if (eel_gconf_get_boolean (CONF_STATE_BM_DOCK_CONTROLS))
+	{
+		gtk_widget_show (controls->edit_frame);
+		gtk_toggle_button_set_active
+			(GTK_TOGGLE_BUTTON (controls->edit_button), TRUE);
+	} 
+	else 
+	{
+		gtk_widget_hide (controls->edit_frame);
+		gtk_toggle_button_set_active
+			(GTK_TOGGLE_BUTTON (controls->edit_button), FALSE);
+	}
+
 	window_dock (window, controls->dialog, 
 		     CONF_STATE_BOOKMARKS_DOCK_WIDTH);
 
@@ -517,14 +593,14 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls,
 		/* b->name is nicely escaped, but we need to unescape
 		 * it here, as GtkCLists and GtkEntries don't know about 
 		 * accels */
-		tmp = strip_uline_accel (b->name);
+		tmp = misc_string_strip_uline_accel (b->name);
 		loc_str = mozilla_utf8_to_locale(tmp);
 		text[0] = loc_str;
 		g_free (tmp);
 		text[1] = g_strdup (b->url);
 		isleaf = TRUE;
 
-		bm_icon = bookmarks_get_siteicon (b->url);
+		bm_icon = favicon_get_pixmap (b->url);
 		pixmap = bm_icon->pixmap;
 		mask = bm_icon->mask;
 
@@ -536,7 +612,7 @@ bookmarks_editor_place_tree_item (BookmarksEditorControls *controls,
 		/* b->name is nicely escaped, but we need to unescape
 		 * it here, as GtkCLists and GtkEntries don't know about 
 		 * accels */
-		tmp = strip_uline_accel (b->name);
+		tmp = misc_string_strip_uline_accel (b->name);
 		loc_str = mozilla_utf8_to_locale(tmp);
 		text[0] = loc_str;
 		g_free (tmp);
@@ -721,7 +797,6 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 	const PixmapData *bm_icon;
 	GdkPixmap *pixmap;
 	GdkBitmap *mask;
-	GtkCTreeNode *parent_node;
 	gchar *loc_str, *tmp;
 		
 	if (!bookmarks_get_tree_item (b, controls)) {
@@ -741,13 +816,13 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 		/* b->name is nicely escaped, but we need to unescape
 		 * it here, as GtkCLists and GtkEntries don't know about 
 		 * accels */
-		tmp = strip_uline_accel (b->name);
+		tmp = misc_string_strip_uline_accel (b->name);
 		loc_str = mozilla_utf8_to_locale(tmp);
 		text[0] = loc_str;
 		g_free (tmp);
 		text[1] = g_strdup (b->url);
 		isleaf = TRUE;
-		bm_icon = bookmarks_get_siteicon (b->url);
+		bm_icon = favicon_get_pixmap (b->url);
 		pixmap = bm_icon->pixmap;
 		mask = bm_icon->mask;;
 		break;
@@ -756,7 +831,7 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 		/* b->name is nicely escaped, but we need to unescape
 		 * it here, as GtkCLists and GtkEntries don't know about 
 		 * accels */
-		tmp = strip_uline_accel (b->name);
+		tmp = misc_string_strip_uline_accel (b->name);
 		loc_str = mozilla_utf8_to_locale(tmp);
 		text[0] = loc_str;
 		g_free (tmp);
@@ -780,11 +855,6 @@ bookmarks_editor_update_tree_item (BookmarksEditorControls *controls, BookmarkIt
 		break;
 	}
 
-	if ((b->parent == NULL) || (b->parent == controls->root_bookmark))
-		parent_node = NULL;
-	else
-		parent_node = bookmarks_get_tree_item (b->parent, controls);
-
 	if (b != controls->root_bookmark) {
 		if (b->alias_of) {
 			/* TODO: use a diferent pixmap for aliases 
@@ -819,7 +889,7 @@ bookmarks_editor_select_bookmark (BookmarksEditorControls *controls,
 	gtk_clist_freeze (GTK_CLIST (controls->ctree));
 
 	/* make sure this node is visible in the bookmarks editor */
-	if (bi->parent == NULL)
+	if ((bi->parent == NULL) && BOOKMARK_ITEM_IS_FOLDER (bi))
 	{
 		bookmarks_editor_set_root (controls, bi);
 		/* thaw the view */
@@ -831,29 +901,32 @@ bookmarks_editor_select_bookmark (BookmarksEditorControls *controls,
 
 	/* make sure the selection is shown */
 	node = bookmarks_get_tree_item (bi, controls);
-	parent = GTK_CTREE_ROW (node)->parent;
-	while (parent != NULL)
+	if (node)
 	{
-		gtk_ctree_expand (GTK_CTREE (controls->ctree),
-				  GTK_CTREE_NODE (parent));
-		parent = GTK_CTREE_ROW (parent)->parent;
+		parent = GTK_CTREE_ROW (node)->parent;
+		while (parent != NULL)
+		{
+			gtk_ctree_expand (GTK_CTREE (controls->ctree),
+					  GTK_CTREE_NODE (parent));
+			parent = GTK_CTREE_ROW (parent)->parent;
+		}
+		
+		/* select it */
+		node = bookmarks_get_tree_item (bi, controls);
+		gtk_ctree_node_moveto (GTK_CTREE (controls->ctree), 
+				       GTK_CTREE_NODE (node), 0, 0, 0);
+		gtk_clist_unselect_all (GTK_CLIST (controls->ctree));
+		gtk_ctree_select (GTK_CTREE (controls->ctree), 
+				  GTK_CTREE_NODE (node));
+		
+		/* find the position (FIXME: necessary?) */
+		row = g_list_position (GTK_CLIST (controls->ctree)->row_list, 
+				       (GList *)node);
+		GTK_CLIST (controls->ctree)->focus_row = row;
+		
+		/* put focus into the name entry */
+		gtk_widget_grab_focus (GTK_WIDGET (controls->name_entry));
 	}
-
-	/* select it */
-	node = bookmarks_get_tree_item (bi, controls);
-	gtk_ctree_node_moveto (GTK_CTREE (controls->ctree), 
-			       GTK_CTREE_NODE (node), 0, 0, 0);
-	gtk_clist_unselect_all (GTK_CLIST (controls->ctree));
-	gtk_ctree_select (GTK_CTREE (controls->ctree), GTK_CTREE_NODE (node));
-
-	/* find the position (FIXME: necessary?) */
-	row = g_list_position (GTK_CLIST (controls->ctree)->row_list, 
-			       (GList *)node);
-	GTK_CLIST (controls->ctree)->focus_row = row;
-
-	/* put focus into the name entry */
-	gtk_widget_grab_focus (GTK_WIDGET (controls->name_entry));
-
 	/* thaw the view */
 	gtk_clist_thaw (GTK_CLIST (controls->ctree));
 }
@@ -864,9 +937,7 @@ bookmarks_editor_add_menubar (BookmarksEditorControls *controls)
 	GtkAccelGroup *accel = GNOME_APP (controls->dialog)->accel_group;
 
 	bookmarks_editor_fill_menu (GTK_MENU_SHELL (controls->menubar),
-				    accel, controls,
-				    &(controls->edit_menuitem),
-				    &(controls->set_default_folder_menuitem));
+				    accel, controls);
 
 	gtk_check_menu_item_set_active 
 				(GTK_CHECK_MENU_ITEM (controls->edit_menuitem),
@@ -880,21 +951,36 @@ bookmarks_editor_popup_menu (BookmarksEditorControls *controls, GtkWidget *w)
 {
 	gint ret;
 	GtkWidget *popup;
+	BookmarkItem* sel = controls->selection;
 
 	if (block_dock_popup_menu) return;
 
 	popup = gtk_menu_new ();
 
 	bookmarks_editor_fill_menu (GTK_MENU_SHELL (popup),
-				    NULL, controls,
-				    NULL, NULL);
+				    NULL, controls);
+
+	block_docking = TRUE;
+	gtk_check_menu_item_set_active 
+		(GTK_CHECK_MENU_ITEM (controls->edit_menuitem),
+		 GTK_WIDGET_VISIBLE(controls->edit_frame));		 
+	gtk_check_menu_item_set_active
+		(GTK_CHECK_MENU_ITEM (controls->dock_menuitem),
+		 (controls->type == DOCKED_BOOKMARKS_EDITOR));
+	block_docking = FALSE;
+	gtk_widget_set_sensitive (controls->set_default_folder_menuitem, 
+				  	sel != NULL && sel->type == BM_FOLDER ? 
+				  	sel != default_bookmarks_root : FALSE);
+		
 
 	block_dock_popup_menu = TRUE;
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), TRUE);
 	ret = gnome_popup_menu_do_popup_modal (popup,
-					       menu_position_under_widget, w,
-					       NULL, NULL);
+			misc_gui_menu_position_under_widget, w, NULL, NULL);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), FALSE);
+	controls->edit_menuitem = NULL;
+	controls->set_default_folder_menuitem = NULL;
+	controls->dock_menuitem = NULL;
 	block_dock_popup_menu = FALSE;
 
 	/* Wait for the menu to be destroyed if a menu item is selected */
@@ -927,9 +1013,7 @@ static char *ugh[] = {
 static void 
 bookmarks_editor_fill_menu (GtkMenuShell *menushell, 
 			    GtkAccelGroup *accel, 
-			    BookmarksEditorControls *controls,
-			    GtkWidget **edit_menuitem,
-			    GtkWidget **set_default_folder_menuitem)
+			    BookmarksEditorControls *controls)
 {
 	GnomeUIInfo file_menu_uiinfo[] =
 	{
@@ -950,9 +1034,17 @@ bookmarks_editor_fill_menu (GtkMenuShell *menushell,
 		  bookmarks_editor_add_alias_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_FILENAME, "../galeon/new_alias_small.xpm",
 		  0, 0, NULL },
+		{ GNOME_APP_UI_ITEM, N_("New a_utobookmarks folder"), NULL,
+		  bookmarks_editor_new_autobm_clicked_cb, controls, NULL,
+		  GNOME_APP_PIXMAP_NONE, NULL,
+		  0, 0, NULL },
 		GNOMEUIINFO_SEPARATOR,
 		GNOMEUIINFO_MENU_OPEN_ITEM
                         (bookmarks_editor_open_clicked_cb, controls),
+		{ GNOME_APP_UI_ITEM, N_("_Import bookmarks..."),
+		  NULL, bookmarks_editor_import_clicked_cb, controls, NULL,
+		  GNOME_APP_PIXMAP_NONE, NULL,
+		  0, 0, NULL },
 		GNOMEUIINFO_MENU_REVERT_ITEM 
 		        (bookmarks_editor_revert_clicked_cb, controls),
 		GNOMEUIINFO_MENU_SAVE_ITEM 
@@ -1066,60 +1158,12 @@ bookmarks_editor_fill_menu (GtkMenuShell *menushell,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
 		GNOMEUIINFO_SEPARATOR,
-		{ GNOME_APP_UI_ITEM, N_("_Dock"), NULL,
+		{ GNOME_APP_UI_TOGGLEITEM, N_("_Dock"), NULL,
 		  bookmarks_editor_show_attach_cb, controls, NULL,
 		  GNOME_APP_PIXMAP_NONE, NULL,
 		  0, 0, NULL },
 		GNOMEUIINFO_END
 	};
-	GnomeUIInfo import_menu_uiinfo[] =
-	{
-		{ GNOME_APP_UI_ITEM, N_("Import from _Galeon's own format..."),
-		  NULL, bookmarks_import_own_activate_cb, controls, NULL,
-		  GNOME_APP_PIXMAP_NONE, NULL,
-		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("Import from _XBEL..."), NULL,
-		  bookmarks_import_xbel_activate_cb, controls, NULL,
-		  GNOME_APP_PIXMAP_NONE, NULL,
-		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("Import from _Netscape bookmarks"),
-		  NULL,
-		  bookmarks_import_netscape_activate_cb, NULL, NULL,
-		  GNOME_APP_PIXMAP_NONE, NULL,
-		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("Import from _Mozilla bookmarks"),
-		  NULL,
-		  bookmarks_import_mozilla_activate_cb, NULL, NULL,
-		  GNOME_APP_PIXMAP_NONE, NULL,
-		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("Import from _other location..."), 
-		  NULL,
-		  bookmarks_import_other_activate_cb, controls, NULL,
-		  GNOME_APP_PIXMAP_NONE, NULL,
-		  0, 0, NULL },
-		GNOMEUIINFO_SEPARATOR,
-		{ GNOME_APP_UI_ITEM, N_("Export bookmarks in X_BEL..."), NULL,
-		  bookmarks_export_xbel_activate_cb, controls, NULL,
-		  GNOME_APP_PIXMAP_NONE, NULL,
-		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("Export to Net_scape bookmarks..."), 
-		  NULL,
-		  bookmarks_export_netscape_activate_cb, controls->dialog, 
-		  NULL,
-		  GNOME_APP_PIXMAP_NONE, NULL,
-		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("Export to Mo_zilla bookmarks..."), 
-		  NULL,
-		  bookmarks_export_mozilla_activate_cb, controls->dialog, NULL,
-		  GNOME_APP_PIXMAP_NONE, NULL,
-		  0, 0, NULL },
-		{ GNOME_APP_UI_ITEM, N_("Export to o_ther location..."), NULL,
-		  bookmarks_export_other_activate_cb, controls, NULL,
-		  GNOME_APP_PIXMAP_NONE, NULL,
-		  0, 0, NULL },
-		GNOMEUIINFO_END
-	};
-
 	/* TODO
 	   GnomeUIInfo help_menu_uiinfo[] =
 	   {
@@ -1131,19 +1175,14 @@ bookmarks_editor_fill_menu (GtkMenuShell *menushell,
 		GNOMEUIINFO_SUBTREE (N_("_File"), file_menu_uiinfo),
 		GNOMEUIINFO_SUBTREE (N_("_Edit"), edit_menu_uiinfo),
 		GNOMEUIINFO_SUBTREE (N_("_View"), view_menu_uiinfo),
-		GNOMEUIINFO_SUBTREE (N_("I_mport / Export"), 
-				     import_menu_uiinfo),
 		/*TODO: GNOMEUIINFO_SUBTREE (N_("_Help"), help_menu_uiinfo),*/
 		GNOMEUIINFO_END
 	};
 	gnome_app_fill_menu (menushell,
 			     menubar_uiinfo, accel, TRUE, 0);
-	if (edit_menuitem)
-		*edit_menuitem = 
-			GTK_WIDGET (view_menu_uiinfo[0].widget);
-	if (set_default_folder_menuitem)
-		*set_default_folder_menuitem = 
-			GTK_WIDGET (edit_menu_uiinfo[6].widget);
+	controls->edit_menuitem = GTK_WIDGET (view_menu_uiinfo[0].widget);
+	controls->dock_menuitem = GTK_WIDGET (view_menu_uiinfo[8].widget);
+	controls->set_default_folder_menuitem = GTK_WIDGET (edit_menu_uiinfo[6].widget);
 }
 
 void
@@ -1164,14 +1203,20 @@ void
 bookmarks_editor_set_root (BookmarksEditorControls *controls, BookmarkItem *b)
 {
 	BookmarkItem *oldroot = controls->root_bookmark;
+	g_return_if_fail (BOOKMARK_ITEM_IS_FOLDER (b));
 	gtk_clist_freeze (GTK_CLIST (controls->ctree));
+
 	controls->root_bookmark = NULL;
+	controls->selection = NULL;
+	controls->last_pressed = NULL;
+
 	if (oldroot != NULL)
 		bookmarks_editor_remove_tree_item_null 
 			(oldroot, controls);
 	gtk_clist_clear (GTK_CLIST (controls->ctree));
 	controls->root_bookmark = b;
 	bookmarks_editor_place_tree_item (controls, controls->root_bookmark);
+
 	gtk_clist_thaw (GTK_CLIST (controls->ctree));
 }
 
@@ -1179,13 +1224,38 @@ void
 bookmarks_editors_refresh (void)
 {
 	GList *l;
-	for (l = bookmarks_editors; l; l = l->next) {
+	for (l = bookmarks_editors; l; l = l->next) 
+	{
 		BookmarksEditorControls *controls = l->data;
 		/* this forces recreating all tree items */
 		bookmarks_editor_set_root (controls, controls->root_bookmark);
 	}
 }
 
+void
+bookmarks_editor_freeze_all (void)
+{
+	GList *li;
+	/* iterate throught the list of all opened editors */
+	for (li = bookmarks_editors; li != NULL; li = li->next) 
+	{
+		BookmarksEditorControls *current_controls = li->data;
+		gtk_clist_freeze (GTK_CLIST (current_controls->ctree));
+	}
+}
+
+void
+bookmarks_editor_thaw_all (void)
+{
+	GList *li;
+	/* iterate throught the list of all opened editors */
+	for (li = bookmarks_editors; li != NULL; li = li->next) 
+	{
+		BookmarksEditorControls *current_controls = li->data;
+		gtk_clist_thaw (GTK_CLIST (current_controls->ctree));
+	}
+}
+
 gboolean
 bookmarks_editor_search (BookmarksEditorControls *controls, gchar *str,
 			 BookmarksSearchIn search_in,
@@ -1285,7 +1355,7 @@ bookmarks_editor_search_matches (BookmarkItem *b, gchar *str,
 	switch (search_method)
 	{
 	case BOOKMARKS_SEARCH_METHOD_CONTAINS:
-		found = tmp && g_strcasestr (tmp, str) != NULL;
+		found = tmp && misc_string_strcasestr (tmp, str) != NULL;
 		break;
 	case BOOKMARKS_SEARCH_METHOD_STARTS_WITH:
 		found = tmp &&
@@ -1307,7 +1377,7 @@ bookmarks_editor_search_matches (BookmarkItem *b, gchar *str,
 	case BOOKMARKS_SEARCH_METHOD_DOES_NOT_CONTAIN:
 		found = TRUE;
 		if (tmp)
-			found = g_strcasestr (tmp, str) == NULL;
+			found = misc_string_strcasestr (tmp, str) == NULL;
 		break;
 	default:
 		g_assert_not_reached ();
@@ -1316,3 +1386,116 @@ bookmarks_editor_search_matches (BookmarkItem *b, gchar *str,
 
 	return found;
 }
+
+/**
+ * This is needed in order to allow only ne of these windows at the same time
+ */
+static BookmarksEditorControls *remove_bookmarks_window = NULL;
+
+BookmarksEditorControls *
+bookmarks_editor_remove_bookmarks_window (GList *bookmarks)
+{
+	if (remove_bookmarks_window == NULL) 
+	{
+		BookmarksEditorControls *controls = NULL;
+		BookmarkItem *fake_root;
+		gint min_to_ask = eel_gconf_get_integer
+			(CONF_BOOKMARKS_REMOVE_CONFIRMATION_MIN);
+		gint number_of_bookmarks = 0;
+		GList *li;
+
+		if (min_to_ask == -1)
+		{
+			/* that means never ask */
+			bookmarks_remove_several (bookmarks);
+			return NULL;
+		}			
+
+		for (li = bookmarks; li != NULL; li = li->next)
+		{
+			number_of_bookmarks += bookmarks_count_tree (li->data);
+		}
+		
+		if (number_of_bookmarks < min_to_ask)
+		{
+			bookmarks_remove_several (bookmarks);
+			return NULL;
+		}
+
+		fake_root = bookmarks_new_bookmark 
+			(BM_FOLDER, TRUE, "Fake root", NULL, NULL, NULL, NULL);
+		
+		/* don't use bookmark_add_child here, because we don't want to 
+		   unparent the bookmarks (yet) */
+		fake_root->list = g_list_copy (bookmarks);
+		
+		controls = bookmarks_editor_controls_init
+			(fake_root, REMOVE_BOOKMARKS_EDITOR);
+		g_return_val_if_fail (controls != NULL, NULL);
+		
+		bookmarks_editors = g_list_prepend (bookmarks_editors,
+						    controls);
+		
+		state_load_column_widths (controls->ctree, "bookmarks_editor");
+		
+		gtk_widget_show (controls->dialog);
+		
+		remove_bookmarks_window = controls;
+	}
+	else
+	{
+		/* just add the new list to the list of bookmarks to be 
+		   deleted, if they are not already there */
+		GList *l;
+		BookmarkItem *fake_root = remove_bookmarks_window->
+			root_bookmark;
+		for (l = bookmarks; l != NULL; l = l->next)
+		{
+			BookmarkItem *b = l->data;
+			if (!bookmarks_bookmark_exists (fake_root, b))
+			{
+				fake_root->list = g_list_append 
+					(fake_root->list, b);
+			}
+		}		
+		
+		/* update the ctree */
+		bookmarks_editor_set_root (remove_bookmarks_window, fake_root);
+	}
+	bookmarks_editor_expand_all_cb (NULL, remove_bookmarks_window);
+	return remove_bookmarks_window;
+}
+
+void
+bookmarks_editor_remove_bookmarks_window_destroy (BookmarksEditorControls
+						  *controls)
+{
+	g_return_if_fail (controls != NULL);
+	g_return_if_fail (controls->type == REMOVE_BOOKMARKS_EDITOR);
+	g_return_if_fail (controls == remove_bookmarks_window);
+
+	/* the destroy callback of the ctree calls this function.
+	   Disconnect that to avoid recursion */
+	gtk_signal_disconnect_by_func (GTK_OBJECT (controls->ctree),
+				       bookmarks_editor_destroy_cb,
+				       controls);
+
+	gtk_clist_freeze (GTK_CLIST(controls->ctree));
+	bookmarks_editor_remove_tree_item (controls, 
+					   controls->root_bookmark);
+	gtk_clist_thaw (GTK_CLIST(controls->ctree));
+	bookmarks_editors = g_list_remove (bookmarks_editors, controls);
+
+	if (controls->clipboard) g_free (controls->clipboard);
+
+	gtk_widget_destroy (controls->dialog);
+	
+	/* the root bookmark is actually a fake bookmark */
+	bookmarks_free_bookmark (controls->root_bookmark);
+
+	/* scrub control structure and free */
+	memset (controls, 0, sizeof (BookmarksEditorControls));
+	g_free (controls);
+	remove_bookmarks_window = NULL;
+}
+
