@@ -1588,19 +1588,18 @@ bookmarks_editor_save_as_clicked_cb (GtkWidget *widget,
 
 	if (show_file_picker (controls->dialog, _("Save as..."),
 			      g_get_home_dir(), NULL, modeSave,
-			      &file, NULL, file_formats, &format_index)) 
-		if (dialog_check_overwrite_file (file, NULL)) {
-			gboolean ok;
-			BookmarkTranslator *selected;
-			selected = g_list_nth 
-				(bookmarks_translators, format_index)->data;
-			ok = selected->save_to_file 
-				(bookmarks_root, default_bookmarks_root, file);
-			if (!ok)
-				gnome_error_dialog 
-					(_("Couldn't save your bookmarks to the "
-					   "specified file!"));
-		}
+			      &file, NULL, file_formats, &format_index)) {
+		gboolean ok;
+		BookmarkTranslator *selected;
+		selected = g_list_nth 
+			(bookmarks_translators, format_index)->data;
+		ok = selected->save_to_file 
+			(bookmarks_root, default_bookmarks_root, file);
+		if (!ok)
+			gnome_error_dialog 
+				(_("Couldn't save your bookmarks to the "
+				   "specified file!"));
+	}
 	g_free (file);
 	g_free (file_formats);
 }
