@@ -476,6 +476,15 @@ bookmarks_editor_ctree_drag_data_received_cb (GtkWidget *widget,
 		{
 		case DND_TARGET_GALEON_BOOKMARK:
 			b = bookmarks_item_from_string (selection_data->data);
+			/* This is a bit hacky... */
+			if (BOOKMARK_ITEM_IS_FOLDER (b) 
+			    && (g_list_length (b->list) == 1))
+			{
+				BookmarkItem *c = b->list->data;
+				bookmark_unparent (c);
+				bookmarks_free_bookmark (b);
+				b = c;
+			}
 			break;
 		case DND_TARGET_TEXT_URI_LIST:
 			utfstr = mozilla_locale_to_utf8(_("New folder"));
