@@ -956,6 +956,7 @@ bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonEmbed *embed,
 	GtkWidget *entry;
 	gchar *s, *tmp, *result, *text, *translated_text;
 	gint dlg_status;
+	GaleonEmbed *tembed;
 
 	entry = gtk_entry_new ();
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), 
@@ -980,13 +981,16 @@ bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonEmbed *embed,
 
 		text = gtk_entry_get_text (GTK_ENTRY (entry));
 
+		/* check if the calling embed was not closed in the meantime */
+		tembed = embed_exists (embed) ? embed : NULL;
+
 		if (text != NULL && strlen (text) > 0)
 		{
 			tmp = g_strndup (bi->smarturl, s - bi->smarturl);
 			translated_text = bookmarks_encode_smb_string (text);
 			result = g_strconcat (tmp, translated_text, s+2, NULL);
-
-			embed_activate_link (embed, newembed, result, state);
+			
+			embed_activate_link (tembed, newembed, result, state);
 
 			g_free (tmp);
 			g_free (translated_text);
@@ -994,7 +998,7 @@ bookmarks_smart_bm_dialog_single (GtkWidget *dialog, GaleonEmbed *embed,
 		}
 		else
 		{
-			embed_activate_link (embed, newembed, bi->url, state);
+			embed_activate_link (tembed, newembed, bi->url, state);
 		}
 	}
 	return dlg_status;
