@@ -18,14 +18,18 @@
 
 #include "galeon.h"
 #include "bookmarks.h"
+#include "bookmarks_editor.h"
+#include "bookmarks_editor_callbacks.h"
 #include "window.h"
 #include "embed.h"
-#include "misc.h"
+#include "misc_general.h"
+#include "misc_gui.h"
 #include "prefs.h"
 #include "dialog.h"
 #include "favicon.h"
-#include "filepicker.h"
+#include "gfilepicker.h"
 #include "mozilla.h"
+#include "bookmarks_io.h"
 
 #include <string.h>
 #include <time.h>
@@ -38,20 +42,6 @@
 #include <gtk/gtkcheckmenuitem.h>
 #include <gdk/gdkkeysyms.h>
 
-void bookmarks_import_mozilla_activate_cb (GtkMenuItem *menuitem,
-					   GaleonWindow *window);
-void bookmarks_import_other_activate_cb (GtkMenuItem *menuitem, 
-					 BookmarksEditorControls *controls);
-static void bookmarks_import_netscape_style (GaleonWindow *window,
-					     GString *fname,
-					     gboolean use_locale);
-void bookmark_export_netscape_message (GaleonWindow *window, gchar *fname);
-static void bookmark_export_netscape_do (gchar *fname, gboolean use_locale);
-static void bookmarks_import_xbel_style (GaleonWindow *window,
-					 const gchar *fname);
-static void bookmarks_import_own_style (GaleonWindow *window, 
-					const gchar *fname);
-
 /* from bookmarks_menus.c */
 extern gint bookmarks_block_accel_update;
 
@@ -62,7 +52,7 @@ bookmarks_handle_link_clicked (GaleonWindow *window, BookmarkItem *bi,
 	g_return_if_fail (bi != NULL);
 	g_return_if_fail (bi->type == BM_SITE);
 	bookmarks_set_visited (bi);
-	if (strstr(bi->url, "%s"))
+	if (bi->smarturl && strstr (bi->smarturl, "%s"))
 	{
 		bookmarks_smart_bm_dialog (window->active_embed, NULL,
 					   bi, state);
@@ -124,8 +114,8 @@ bookmarks_button_release_event_cb (GtkWidget *widget, GdkEventButton *event,
 		if (GTK_IS_MENU_SHELL (menu))
 			gtk_menu_shell_deactivate (GTK_MENU_SHELL(menu));
 		bookmarks_handle_link_clicked (window, bi, 
-				mouse_state_to_link_state (event->button,
-							   event->state));
+				misc_general_mouse_state_to_link_state (
+						event->button, event->state));
 	}
 	else if (bi->type == BM_AUTOBOOKMARKS || bi->type == BM_FOLDER)
 	{
@@ -271,7 +261,7 @@ bookmarks_copy_link_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 	g_return_if_fail (bi != NULL);
 	g_return_if_fail (bi->type == BM_SITE);
 
-	copy_to_clipboard (bi->url, window->active_embed);
+	embed_copy_text_to_clipboard (bi->url, window->active_embed);
 }
 
 void
@@ -284,32 +274,18 @@ bookmarks_add_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 void
 bookmarks_add_folder_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 {
-	BookmarkItem *new;
-	GaleonWindow *par_gwin = window_from_widget (widget);
-	GtkWidget *par_win = par_gwin ? par_gwin->WMain : NULL;
-	gchar* new_folder_name = NULL;
+	gchar *name;
+	GaleonWindow *window;
 	
-	g_return_if_fail (bi != NULL);
-	g_return_if_fail ((bi->type == BM_FOLDER) 
-			  || (bi->type == BM_AUTOBOOKMARKS));
+	/* get parent window */
+	window = window_from_widget (widget);
+	return_if_not_window (window);
 
-	new_folder_name = mozilla_locale_to_utf8(_("New folder"));
-	new = bookmarks_new_bookmark (BM_FOLDER, TRUE, new_folder_name,
-				      NULL, NULL, NULL, NULL);
-	g_free(new_folder_name);
-	new->time_added = time (NULL);
+	name = mozilla_locale_to_utf8 (_("New folder"));
 
-	bi->list = g_list_append (bi->list, new);
-	new->parent = bi;
-
-	bookmarks_dirty = TRUE;
-	bookmarks_toolbars_check_update (new);
-
-	gnome_request_dialog 
-		(FALSE, _("Please enter the name of the folder"),
-		 _("New folder"), 100,
-		 GTK_SIGNAL_FUNC (bookmarks_string_request_callback),
-		 new, par_win ? GTK_WINDOW (par_win) : NULL);
+	/* add the folder */
+	bookmarks_add_folder (name, bi, GTK_WINDOW (window->wmain),
+			      BOOKMARK_ADD_FORCE_PROMPT_TITLE);
 }
 
 void
@@ -374,7 +350,10 @@ bookmarks_create_toolbar_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 	bi->create_toolbar = !bi->create_toolbar;
 	/* called twice intentionally */
 	bookmarks_toolbars_check_update (bi);
-	bookmarks_updated ();
+	if (!gtk_object_get_data (GTK_OBJECT (widget->parent), "in_editor"))
+	{
+		bookmarks_updated ();
+	}
 }
 
 void
@@ -420,7 +399,11 @@ bookmarks_set_as_default_activate_cb (GtkWidget *widget, BookmarkItem *bi)
 		default_bookmarks_root = bi;
 		bookmarks_toolbars_check_update (bi);
 	}
-	bookmarks_updated ();
+	if (widget &&
+	    !gtk_object_get_data (GTK_OBJECT (widget->parent), "in_editor"))
+	{
+		bookmarks_updated ();
+	}
 }
 
 void
@@ -437,14 +420,10 @@ void
 bookmarks_edit_activate_cb (GtkWidget *menuitem, BookmarkItem *bm)
 {
 	BookmarksEditorControls *controls;
-	GtkWidget *popup;
-	GaleonWindow *window;
-
-	/* get GaleonWindow */
-	popup = gtk_widget_get_toplevel (menuitem); 
-	window = (GaleonWindow *) gtk_object_get_data (GTK_OBJECT(popup), 
-						       "GaleonWindow");
+	GaleonWindow *window = window_from_widget (menuitem);
 
+	g_return_if_fail (window != NULL);
+	
 	controls = bookmarks_editor_show_dialog (window);
 	bookmarks_editor_select_bookmark (controls, bm);
 }
@@ -457,9 +436,9 @@ void
 bookmarks_remove_activate_cb (GtkWidget *menuitem, 
 			      BookmarkItem *bm)
 {
-	gboolean autobookmarks_deleted = FALSE;
-	
+	BookmarksEditorControls *remove_window;
 	GaleonWindow *window = window_from_widget (menuitem);
+	GList *l = g_list_prepend (NULL, bm);
 
 	/* Make sure that the bookmarks menu has disapeared from the
 	   screen, otherwise the menu can hide the dialog (and it would be 
@@ -476,37 +455,120 @@ bookmarks_remove_activate_cb (GtkWidget *menuitem,
 				(GTK_MENU_SHELL (item->parent));
 	}
 
-	if (bm->list /* don't ask about deleting an empty folder or an alias */
-	    && !bm->alias_of
-	    && !dialog_ask_question 
-	    (_("Removing this category will remove all its children.\n"
-	       "Are you sure that you want to continue?"),
-	     window->WMain))
+	remove_window = bookmarks_editor_remove_bookmarks_window (l);
+	g_list_free (l);
+	/* update the UI as soon as the dialog is destroyed */
+	if (remove_window)
 	{
-		return;
+		gtk_signal_connect (GTK_OBJECT (remove_window->dialog), "destroy", 
+				    GTK_SIGNAL_FUNC (bookmarks_updated), NULL);
+	}
+	else
+	{
+		/* they were removed without confirmation */
+		bookmarks_updated ();
+	}
+}
+
+/**
+ * bookmarks_export_activate_cb: export current bookmark to a file
+ */
+void
+bookmarks_export_activate_cb (GtkWidget *menuitem, 
+			      BookmarkItem *bm)
+{
+	gboolean ret;
+	gchar *file = NULL;
+	GaleonWindow *window = window_from_widget (menuitem);
+
+	/* Make sure that the bookmarks menu has disapeared from the
+	   screen, otherwise the menu can hide the dialog (and it would be 
+	   very bad) */
+	if (window) 
+	{
+		GtkWidget *item = gtk_menu_get_attach_widget 
+			(GTK_MENU (window->bookmarks_menu));		
+		gtk_menu_popdown (GTK_MENU (window->bookmarks_menu));
+		gtk_menu_item_deselect (GTK_MENU_ITEM (item));
+		/* the menubar */ 
+		if (GTK_IS_MENU_SHELL (item->parent))
+			gtk_menu_shell_deactivate 
+				(GTK_MENU_SHELL (item->parent));
+	}
+	
+	ret = show_file_picker (NULL, _("Select the destination filename"),
+				g_get_home_dir (), "bookmark.xbel", modeSave,
+				&file, NULL, NULL, NULL);
+
+	if (ret)
+	{
+		bookmarks_default_translator->save_to_file (bm, 
+				default_bookmarks_root, file);
 	}
 
-	bookmarks_toolbars_check_update (bm);
-	if (bm->type == BM_AUTOBOOKMARKS)
+	if (file) g_free (file);
+}
+
+/**
+ * bookmarks_import_activate_cb: import a bookmarks file to this folder
+ */
+void
+bookmarks_import_activate_cb (GtkWidget *menuitem, 
+			      BookmarkItem *bm)
+{
+	gboolean ret;
+	gchar *file = NULL;
+	GaleonWindow *window = window_from_widget (menuitem);
+
+	/* Make sure that the bookmarks menu has disapeared from the
+	   screen, otherwise the menu can hide the dialog (and it would be 
+	   very bad) */
+	if (window) 
 	{
-		autobookmarks_deleted = TRUE;
-	}			
-	bookmarks_editor_remove_tree_items (bm);
-	bookmarks_remove_recursively (bm);
-	if (autobookmarks_deleted)
-	{
-		autobookmarks_root = NULL;
+		GtkWidget *item = gtk_menu_get_attach_widget 
+			(GTK_MENU (window->bookmarks_menu));		
+		gtk_menu_popdown (GTK_MENU (window->bookmarks_menu));
+		gtk_menu_item_deselect (GTK_MENU_ITEM (item));
+		/* the menubar */ 
+		if (GTK_IS_MENU_SHELL (item->parent))
+			gtk_menu_shell_deactivate 
+				(GTK_MENU_SHELL (item->parent));
 	}
-	bookmarks_updated ();
+	
+	ret = show_file_picker (NULL, _("Select the file to open"),
+				g_get_home_dir (), NULL, modeOpen,
+				&file, NULL, NULL, NULL);
+
+	if (ret)
+	{
+		BookmarkItem *imported = 
+			bookmarks_default_translator->load_from_file (file, NULL);
+		if (imported)
+		{
+			bookmarks_merge_trees (bm, imported);
+			bookmarks_free_bookmark (imported);
+		}
+		bookmarks_dirty = TRUE;
+		bookmarks_toolbars_check_update (bm);
+		if (!gtk_object_get_data (GTK_OBJECT (menuitem->parent),
+					 "in_editor"))
+		{
+			bookmarks_updated ();
+		}
+		else
+		{
+			bookmarks_editors_refresh ();
+		}
+	}
+
+	if (file) g_free (file);
 }
 
 void
 bookmarks_file_bookmark_cb (GtkWidget *widget, BookmarkItem *p)
 {
-	gchar *loc_str;
-	BookmarkItem *new;	
-	GaleonEmbed *embed;
 	GaleonWindow *window;
+	GaleonEmbed *embed;
 
 	window = window_from_widget (widget);
 	return_if_not_window (window);
@@ -514,231 +576,9 @@ bookmarks_file_bookmark_cb (GtkWidget *widget, BookmarkItem *p)
 	embed = window->active_embed;
 	return_if_not_embed (embed);
 
-	new = bookmarks_new_bookmark (BM_SITE, TRUE, embed->site_title_utf8, 
-				      embed->site_location, NULL, NULL, NULL);
-	new->time_added = time (NULL);
-
-	p->list = g_list_append (p->list, new);
-	new->parent = p;
-	bookmarks_dirty = TRUE;
-	bookmarks_toolbars_check_update (new);
-
-	if (eel_gconf_get_integer(CONF_BOOKMARKS_TITLE) == 
-	    BOOKMARK_PAGE_TITLE)
-	{
-		if (new->url && 
-		    eel_gconf_get_boolean (CONF_BOOKMARKS_FAVICONS_ENABLED))
-			favicon_get_siteicon (embed, new, NULL);
-
-		bookmarks_editor_place_tree_items (new);
-		bookmarks_toolbars_check_update (new);
-		bookmarks_updated ();
-	} else {
-		loc_str = mozilla_utf8_to_locale (new->name);
-		gnome_request_dialog 
-			(FALSE, _("Please enter the name of the bookmark"),
-			 loc_str, 100,
-			 GTK_SIGNAL_FUNC (bookmarks_string_request_callback),
-			 new, GTK_WINDOW (window->WMain));
-		g_free (loc_str);
-	}
-}
-
-/* import given netscape-compatible bookmark filename with pass/fail dialog */
-static void
-bookmarks_import_netscape_style (GaleonWindow *window, GString *fname,
-				 gboolean use_locale)
-{
-	/* NB: used to use gnome_app_message here, but there's not a 
-	 * good choice of window to associate the message with, so 
-	 * forcing a dialog seems like a better idea -- MattA 30-12-2000 */
-	if (netscape_import_bookmarks(fname, use_locale))
-	{
-		gnome_ok_dialog (_("Importing bookmarks failed"));
-	}
-	else
-	{
-		gnome_ok_dialog (_("Bookmarks successfully imported/merged"));
-	}
-	bookmarks_tb_dirty = TRUE;
-	bookmarks_updated ();
-}
-
-static void
-bookmarks_import_xbel_style (GaleonWindow *window, const gchar *fname)
-{
-	BookmarkItem *imported = xbel_import_bookmarks (fname);
-	if (imported) {
-		bookmarks_merge_trees (bookmarks_root, imported);
-		bookmarks_free_bookmark (imported);
-		gnome_ok_dialog (_("Bookmarks successfully imported/merged"));
-	} else {
-		gnome_ok_dialog (_("Importing bookmarks failed"));
-	}
-	bookmarks_tb_dirty = TRUE;
-	bookmarks_updated ();
-	bookmarks_editors_refresh ();
-}
-
-static void
-bookmarks_import_own_style (GaleonWindow *window, const gchar *fname)
-{
-	BookmarkItem *imported = bookmarks_load_from ((gchar *) fname);
-	if (imported) {
-		bookmarks_merge_trees (bookmarks_root, imported);
-		bookmarks_free_bookmark (imported);
-		gnome_ok_dialog (_("Bookmarks successfully imported/merged"));
-	} else {
-		gnome_ok_dialog (_("Importing bookmarks failed"));
-	}
-	bookmarks_updated ();
-	bookmarks_editors_refresh ();
-}
-
-void
-bookmarks_import_netscape_activate_cb (GtkMenuItem *menuitem,
-				       GaleonWindow *window)
-{
-	GString *fname = g_string_new (NULL);
-
-	g_string_assign (fname, g_get_home_dir ());
-	g_string_append (fname, NETSCAPE_BOOKMARKS);
-       	
-	bookmarks_import_netscape_style (window, fname, TRUE);
-
-	g_string_free (fname, TRUE);
-}
-
-void
-bookmarks_import_mozilla_activate_cb (GtkMenuItem *menuitem,
-				      GaleonWindow *window)
-{
-	GString * fname = g_string_new (NULL);
-	char *prefs_dir = get_mozilla_prefs_directory ();
-
-	if (!prefs_dir) return;
-
-	g_string_assign (fname, prefs_dir);
-	g_string_append (fname, "/bookmarks.html");
-
-	bookmarks_import_netscape_style (window, fname, FALSE);
-
-	g_string_free (fname, TRUE);
-	g_free (prefs_dir);
-}
-
-void
-bookmarks_import_other_activate_cb (GtkMenuItem *menuitem, 
-				    BookmarksEditorControls *controls)
-{
-	gchar *file = NULL;
-
-	if (show_file_picker (controls->dialog, _("Import from..."),
-			      g_get_home_dir(), NULL, modeOpen,
-			      &file, NULL)) {
-		GString *temp = g_string_new (file);
-		bookmarks_import_netscape_style (NULL, temp, FALSE);
-		g_string_free (temp, TRUE);
-	}
-
-	g_free (file);
-}
-
-void
-bookmarks_import_xbel_activate_cb (GtkMenuItem *menuitem, 
-				   BookmarksEditorControls *controls)
-{
-	gchar *file = NULL;
-
-	if (show_file_picker (controls->dialog, _("Import from..."),
-			      g_get_home_dir(), NULL, modeOpen,
-			      &file, NULL))
-		bookmarks_import_xbel_style (NULL, file);
-	
-	g_free (file);
-}
-
-void
-bookmarks_import_own_activate_cb (GtkMenuItem *menuitem, 
-				  BookmarksEditorControls *controls)
-{
-	gchar *file = NULL;
-
-	if (show_file_picker (controls->dialog, _("Import from..."),
-			      g_get_home_dir(), NULL, modeOpen,
-			      &file, NULL))
-		bookmarks_import_own_style (NULL, file);
-	
-	g_free (file);
-}
-
-void
-bookmarks_export_netscape_activate_cb (GtkMenuItem *menuitem,
-				       GtkWidget *widget)
-{
-	gchar *fname = g_strconcat (g_get_home_dir(), NETSCAPE_BOOKMARKS,
-				    NULL);
-       	
-	if (dialog_check_overwrite_file (fname, widget))
-		bookmark_export_netscape_do (fname, TRUE);
-
-	g_free (fname);
-}
-
-void
-bookmarks_export_mozilla_activate_cb (GtkMenuItem *menuitem,
-				      GtkWidget *widget)
-{
-	char *fname;
-	char *prefs_dir = get_mozilla_prefs_directory();
-
-	fname = g_strconcat (prefs_dir,"/bookmarks.html", NULL);
-	if (dialog_check_overwrite_file (fname, widget))
-		bookmark_export_netscape_do (fname, FALSE);
-
-	g_free (fname);
-	g_free (prefs_dir);
-}
-
-void
-bookmarks_export_other_activate_cb (GtkMenuItem *menuitem, 
-				    BookmarksEditorControls *controls)
-{
-	gchar *file = NULL;
-
-	if (show_file_picker (controls->dialog, _("Export to..."),
-			      g_get_home_dir(), NULL, modeSave,
-			      &file, NULL))
-	{
-		if (dialog_check_overwrite_file (file, NULL))
-			bookmark_export_netscape_do (file, FALSE);
-		g_free (file);
-	}
-}
-
-void
-bookmarks_export_xbel_activate_cb (GtkMenuItem *menuitem, 
-				   BookmarksEditorControls *controls)
-{
-	gchar *file = NULL;
-
-	if (show_file_picker (controls->dialog, _("Export in XBEL as..."),
-			      g_get_home_dir (), NULL, modeSave,
-			      &file, NULL))
-	{
-		if (dialog_check_overwrite_file (file, NULL))
-			xbel_export_bookmarks (file, bookmarks_root);
-		g_free (file);
-	}
-}
-
-static void 
-bookmark_export_netscape_do (gchar *fname, gboolean use_locale)
-{
-	if (netscape_export_bookmarks (fname, bookmarks_root, use_locale))
-		gnome_ok_dialog (_("Exporting bookmarks failed"));
-	else 
-		gnome_ok_dialog (_("Bookmarks successfully exported"));
+	/* add the bookmark */
+	bookmarks_add_bookmark (embed->title_utf8, embed->location, p,
+				GTK_WINDOW (window->wmain), 0);
 }
 
 void
@@ -782,8 +622,9 @@ bookmarks_remove_accel_cb (GtkWidget *widget, GtkAccelGroup *accel_group,
 {
 	if (bookmarks_block_accel_update > 0) return;
 
-	/* bookmarks will need to be updated */
-	bookmarks_dirty = TRUE;
+	/* check whether bookmarks will need to be updated */
+	if (!(bi->accel_key == 0 && bi->accel_mods == 0))
+		bookmarks_dirty = TRUE;
 
 	/* update bookmark */
 	bi->accel_key = 0;
@@ -794,44 +635,100 @@ bookmarks_remove_accel_cb (GtkWidget *widget, GtkAccelGroup *accel_group,
 }
 
 void
-bookmarks_ensure_accel_cb (GtkWidget *widget, BookmarkItem *bm)
+bookmarks_select_cb (GtkWidget *widget, BookmarkItem *bm)
 {
+	GaleonWindow *window = window_from_widget (widget);
 	GtkMenu *menu;
 	GtkWidget *l;
 	gchar *str;
 
+	g_assert (window);	
+
 	if (bookmarks_block_accel_update > 0) return;
 
-	g_return_if_fail (GTK_IS_MENU_ITEM (widget));
-	g_return_if_fail (GTK_IS_MENU (widget->parent));
+	g_assert (window != NULL);
 
-	menu = GTK_MENU (widget->parent);
+	if ((l = gtk_object_get_data (GTK_OBJECT (widget), "accel_label")))
+	{
+		g_return_if_fail (GTK_IS_MENU_ITEM (widget));
+		g_return_if_fail (GTK_IS_MENU (widget->parent));
 
-	bookmarks_block_accel_update++;
-	gtk_widget_add_accelerator (widget, "activate", menu->accel_group, 
-				    bm->accel_key, bm->accel_mods, 
-				    GTK_ACCEL_VISIBLE);
-	bookmarks_block_accel_update--;
+		menu = GTK_MENU (widget->parent);
+
+		bookmarks_block_accel_update++;
+		if (bm->accel_key != GDK_VoidSymbol)
+			gtk_widget_add_accelerator (widget, "activate",
+						    menu->accel_group, 
+						    bm->accel_key, 
+						    bm->accel_mods, 
+						    GTK_ACCEL_VISIBLE);
+		bookmarks_block_accel_update--;
 	
-	/* update accel label too, 
-	 * FIXME: this is not really neccesary 90% of time */
+		/* update accel label too, 
+		 * FIXME: this is not really neccesary 90% of time */
 	
-	l = gtk_object_get_data (GTK_OBJECT (widget), "accel_label");
-	g_return_if_fail (l != NULL);
+		str = misc_gui_string_from_accel_data (widget, bm->accel_key,
+						       bm->accel_mods);
+		gtk_label_set_text (GTK_LABEL (l), str);
+		g_free (str);
+	}
 
-	str = string_from_accel_data (widget, bm->accel_key, bm->accel_mods);
-	gtk_label_set_text (GTK_LABEL (l), str);
-	g_free (str);
+	/* update statusbar */
+	if (eel_gconf_get_integer (CONF_BOOKMARKS_TOOLTIPS) == 1)
+	{
+		GtkWidget *entry =
+			gtk_object_get_data (GTK_OBJECT (widget), "entry");
+		gchar *msg = NULL;
+		gint i;
+
+		if (entry)
+		{
+			gchar *text = gtk_entry_get_text (GTK_ENTRY (entry));
+
+			if (text != NULL && strlen (text) > 0)
+			{
+		                /* get a completed url */
+				gchar *trans =
+					bookmarks_encode_smb_string (text);
+				msg = bookmarks_substitute_argument (bm, trans);
+				g_free (trans);
+			}
+			else if (bm->url != NULL && strlen (bm->url) > 0)
+			{
+				msg = g_strdup (bm->url);
+			}
+			else
+			{
+				msg = g_strdup (bm->smarturl);
+			}
+		}
+		else
+		{
+			msg = g_strdup (bm->url);
+		}
+
+		/* get rid of newline chars */
+		i = 0;
+		while (msg[i] != '\0')
+		{
+			if (msg[i] == '\n')
+				msg[i] = ' ';
+			i++;
+		}
+		
+		gnome_appbar_push (GNOME_APPBAR (window->appbar), msg);
+		g_free (msg);
+	}
 }
 
 void
-bookmarks_remove_dummy_accel_cb (GtkWidget *widget, GtkAccelGroup *accel_group, 
-			         guint accel_key, GdkModifierType accel_mods, 
-			         BookmarkItem *bi)
+bookmarks_deselect_cb (GtkWidget *widget, BookmarkItem *bm)
 {
-	if (bookmarks_block_accel_update > 0) return;
+	GaleonWindow *window = window_from_widget (widget);
 
-	/* update bookmark */
-	bi->accel_key = 0;
-	bi->accel_mods = 0;
+	g_assert (window);
+
+	if (eel_gconf_get_integer (CONF_BOOKMARKS_TOOLTIPS) == 1)
+		gnome_appbar_pop (GNOME_APPBAR (window->appbar));
 }
+
