@@ -20,9 +20,7 @@
 extern "C" {
 #endif
 
-#include <glade.h>
-
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 
 gboolean dialog_ask_question (const gchar *question, GtkWidget *parent);
 gboolean dialog_check_overwrite_file (const gchar *fname, GtkWidget *parent);
