@@ -70,7 +70,7 @@ static void embed_set_save_message (GaleonEmbed *embed, char *path,
 				    gboolean result);
 static void embed_select_new_page (GaleonEmbed *embed);
 static gint check_mozilla_version (void);
-static gint read_mozilla_version (void);
+static gchar *read_mozilla_version (void);
 
 /** The global list of all GaleonEmbed structures */
 GList *all_embeds = NULL;
@@ -139,7 +139,7 @@ embed_create_in_window_chrome (GaleonWindow *window,
 	/* keep create flags */
 	embed->create_flags = flags;
 
-	if (flags & EMBED_CREATE_DONT_AUTOSAVE_INITIALLY)
+	if (flags & EMBED_CREATE_DONT_AUTOSAVE_SESSION)
 		embed->dont_autosave_session = TRUE;
 	else embed->dont_autosave_session = FALSE;
 
@@ -180,13 +180,6 @@ embed_create_in_window_chrome (GaleonWindow *window,
 	{
 		embed_set_visibility (embed, TRUE);
 		embed_load_url (embed, url);
-
-		/* set modified location, so the location will show up
-		 * in the entry */
-		embed->modified_location = g_strdup (url);
-		embed->location_is_modified = TRUE;
-		if (embed->is_active)
-			window_update_location_entry_text (window);
 	}
 	else if (!(flags & EMBED_CREATE_DONT_SHOW))
 	{
@@ -272,9 +265,8 @@ embed_create_no_window (void)
 
 	/* no content for the location or title */
 	embed->location = NULL;
-	embed->realurl = NULL;
+	embed->parent_window = NULL;
 	embed->modified_location = NULL;
-	embed->location_is_modified = FALSE;
 	embed->title = g_strdup (_("Untitled"));
 	embed->title_utf8 = mozilla_locale_to_utf8 (embed->title);
 
@@ -331,6 +323,11 @@ embed_create_no_window (void)
 	embed->gesture_press_signal = 0;
 	embed->gesture_release_signal = 0;
 
+	/* clear scrolling stuff */
+	embed->scroll_motion_signal = 0;
+	embed->scroll_release_signal = 0;
+	embed->scroll_timeout_id = 0;
+
 	/* done! */
 	return embed;
 }
@@ -456,23 +453,8 @@ embed_load_url (GaleonEmbed *embed, const gchar *url)
 
 	clean_url = misc_string_remove_outside_whitespace (url);
 
-	/* initialise embed whenever a document is first loaded */
-	if (embed->wrapper == NULL)
-	{
-		embed_wrapper_init (embed);
-
-		/* hack to work around a bunch of problems when nothing
-		 * is loaded */
-		embed->realurl = g_strdup (clean_url);
-		gtk_moz_embed_load_url (GTK_MOZ_EMBED (embed->mozembed),
-					"about:blank");
-	}
-	else
-	{
-		/* load the URL */
-		gtk_moz_embed_load_url (GTK_MOZ_EMBED(embed->mozembed),
-					clean_url);
-	}
+	/* load the URL */
+	gtk_moz_embed_load_url (GTK_MOZ_EMBED (embed->mozembed), clean_url);
 
 	/* free allocated string */
 	g_free (clean_url);
@@ -550,7 +532,7 @@ embed_activate_link (GaleonEmbed *previous, GaleonEmbed **new_embed,
 		/* left button (or no button) */
 		if (!previous) {
 			new = previous = embed_create_in_window (NULL, NULL,
-				"about:blank", EMBED_CREATE_RAISE_WINDOW);
+				"about:blank", 0);
 		}
 		if (shifted) {
 			downloader_save_link (previous, url);
@@ -591,7 +573,6 @@ embed_activate_link (GaleonEmbed *previous, GaleonEmbed **new_embed,
 				{
 					new = embed_create_after_embed (
 						previous, !tabbed_mode, url,
-						EMBED_CREATE_RAISE_WINDOW |
 						EMBED_CREATE_GRAB_FOCUS |
 						EMBED_CREATE_NEVER_JUMP);
 				}
@@ -599,7 +580,6 @@ embed_activate_link (GaleonEmbed *previous, GaleonEmbed **new_embed,
 				{
 					new = embed_create_after_embed (
 						previous, !tabbed_mode, url,
-						EMBED_CREATE_RAISE_WINDOW |
 						EMBED_CREATE_GRAB_FOCUS |
 						EMBED_CREATE_FORCE_JUMP);
 				}
@@ -609,7 +589,6 @@ embed_activate_link (GaleonEmbed *previous, GaleonEmbed **new_embed,
 			{
 				new = embed_create_after_embed (previous,
 					!tabbed_mode, url,
-					EMBED_CREATE_RAISE_WINDOW |
 					EMBED_CREATE_GRAB_FOCUS);
 			}
 		}
@@ -753,11 +732,10 @@ embed_startup_init ()
 /**
  * read_mozilla_version: extract mozilla version from mozilla prefs file
  */
-static gint
+static gchar *
 read_mozilla_version (void)
 {
-	gchar agentstr[160], *p, *endptr;
-	gint ver= 0, v;
+	gchar agentstr[160], *p, *endptr, *ver;
 	gchar *preffile;
 	FILE *f;
 
@@ -769,7 +747,7 @@ read_mozilla_version (void)
 	{
 		g_warning ("error opening %s: %s", preffile, strerror (errno));
 		g_free (preffile);
-		return 0;
+		return NULL;
 	}
 
 	while ((p = fgets (agentstr, 160, f)) != NULL)
@@ -780,32 +758,20 @@ read_mozilla_version (void)
 	}
 	fclose(f);
 
-	if (p == NULL)
+ 	ver = NULL;
+ 	if (p == NULL || p + 3 == '\0')
 	{
 		g_warning ("Mozilla version not found in %s", preffile);
 	}
 	else
 	{
 		p += 3;
-		v = strtol (p, &endptr, 10);
-		ver = v << 24;
-		if (*endptr == '.')
-		{
-			p = endptr + 1;
-			v = strtol (p, &endptr, 10);
-			ver += v << 16;
-			if (*endptr == '.')
-			{
-				p = endptr + 1;
-				v = strtol (p, &endptr, 10);
-				ver += v << 8;
-			}
-		}
-		if (*endptr == '+')
-		{
-			ver++;
-		}
-	}
+ 		if ((endptr = strchr(p, '"')) != NULL);
+  		{
+ 			*endptr = '\0';
+  		}
+ 		ver = g_strdup (p);
+  	}
 
 	g_free (preffile);
 	return ver;
@@ -819,23 +785,28 @@ read_mozilla_version (void)
 static gint
 check_mozilla_version (void)
 {
-	mozilla_version = read_mozilla_version ();
-
-	if (MOZILLA_VERSION != mozilla_version)
+	gchar *ver;
+	
+	ver = read_mozilla_version ();
+	
+	if (ver == NULL)
 	{
-		gchar *questionStr, *rememberStr, *verMatchStr;
-		gchar *version1_str, *version2_str;
+		ver = g_strdup ("Unknown");
+	}
+	
+	/* FIXME: also include matching with the NS_BUILD_ID , after figuring
+	 * how to get it on runtime :) */
+	if (strcmp(MOZILLA_VERSION, ver))
+	{
+		gchar *questionStr, *rememberStr, *verMatchStr, *runningverStr;
 		gboolean continuev=0, rememberv=0;
 
-		g_warning ("compiled MOZILLA_VERSION ("VERSIONFMT") !="
-			" detected mozilla_version ("VERSIONFMT")",
-			VERSIONFMTARGS (MOZILLA_VERSION),
-			VERSIONFMTARGS (mozilla_version));
+		g_warning ("compiled mozilla version "MOZILLA_VERSION" != "
+			   "detected mozilla_version %s", ver);
 
-		verMatchStr = g_strdup_printf ("%xv%x", MOZILLA_VERSION,
-						mozilla_version);
-		rememberStr = eel_gconf_get_string 
-				(CONF_MOZ_MISMATCH);
+		verMatchStr = g_strdup (MOZILLA_VERSION);
+		runningverStr = g_strdup (ver);
+		rememberStr = eel_gconf_get_string (CONF_MOZ_MISMATCH);
 
 		if (rememberStr != NULL)
 		{
@@ -844,26 +815,20 @@ check_mozilla_version (void)
 			if (ignore == 0)
 			{
 				g_free (verMatchStr);
+				g_free (runningverStr);
+				g_free (ver);
 				return eel_gconf_get_integer
 				    (CONF_MOZ_MISMATCH_ACTION);
 			}
 		}
 
-		version1_str = g_strdup_printf (VERSIONFMT,
-					VERSIONFMTARGS (MOZILLA_VERSION));
-		version2_str = g_strdup_printf (VERSIONFMT,
-					VERSIONFMTARGS (mozilla_version));
-
 		questionStr = g_strdup_printf (_(
 "The version of Mozilla Galeon was compiled against (%s) "
 "does not match the detected version (%s).\n\n"
 "You may experience segfaults and other oddities.  Please test "
 "with a matching version of Mozilla before reporting any bugs.\n\n"
 "Continue anyway?"),
-			version1_str, version2_str);
-
-		g_free (version1_str);
-		g_free (version2_str);
+			verMatchStr, runningverStr);
 
 		continuev = dialog_confirm_check (NULL, _("Version mismatch!"),
 					questionStr, _("Don't ask me again"),
@@ -880,8 +845,10 @@ check_mozilla_version (void)
 				 !continuev);
 		}
 		g_free (verMatchStr);
+		g_free (runningverStr);
 		return !continuev;
 	}
+	g_free (ver);
 	return 0;
 }
 
@@ -1120,7 +1087,7 @@ embed_save_url (GaleonEmbed *embed, gchar *target,
 	if (!EMBED_IN_NAUTILUS (embed))
 #endif
 	{
-		if (embed->parent_window)
+		if (embed && embed->parent_window)
 			parent = embed->parent_window->wmain;
 	}
 
@@ -1298,13 +1265,12 @@ void embed_view_source (GaleonEmbed *embed, gboolean main,
 	}
 	else
 	{
-		gchar *vs_url;
-
-		vs_url = embed_get_vs_url_from_string (url);
-		embed_create_after_embed (embed, new_window, vs_url,
-					  EMBED_CREATE_FORCE_JUMP |
-					  EMBED_CREATE_RAISE_WINDOW);
-		g_free (vs_url);
+		DisplayType type = DISPLAY_AS_SOURCE;
+		GaleonEmbed *vs_embed;
+		vs_embed = embed_create_after_embed(embed, new_window,
+						    "about:blank",
+						    EMBED_CREATE_FORCE_JUMP);
+		mozilla_copy_page (vs_embed, embed, type);
 	}
 	g_free (url);
 }
@@ -1350,6 +1316,9 @@ embed_close (GaleonEmbed *embed)
 
 	embed->being_closed = TRUE;
 
+	/* make sure we're not still in print preview mode or bad things happen fast */
+	mozilla_print_preview_close(embed);
+
 	/* stop any pending loads */
 	gtk_moz_embed_stop_load (GTK_MOZ_EMBED (embed->mozembed));
 
@@ -1375,6 +1344,10 @@ embed_close (GaleonEmbed *embed)
 	if (embed->gesture_release_signal)
 		mozembed_gesture_disconnect (embed);
 
+	/* if we're scrolling, clean up */
+	if (embed->scroll_release_signal)
+		mozembed_scroll_disconnect (embed);
+
 	/* destroy the embedding widget -- this will also destroy 
 	 * the notebook tab and its label, and remove it from 
 	 * the relevant lists */
@@ -1404,8 +1377,7 @@ embed_open_frame (GaleonEmbed *embed, gboolean same_embed, gboolean new_window)
 	else 
 	{
 		embed_create_after_embed (embed, new_window, url,
-					  EMBED_CREATE_FORCE_JUMP |
-					  EMBED_CREATE_RAISE_WINDOW);
+					  EMBED_CREATE_FORCE_JUMP);
 	}
 
 	/* free */
@@ -1422,53 +1394,6 @@ embed_reload (GaleonEmbed *embed, int flags)
 			      flags);
 }
 
-/**
- * embed_update_location: updates the embed's location variable, and also
- * the parent window's location entry, if appropriate.
- */
-void
-embed_update_location (GaleonEmbed *embed)
-{
-	/* set the actual location */
-	if (embed->location) g_free (embed->location);
-	embed->location = mozilla_get_real_url (embed);
-
-	/* clear modified location, if about:blank wasn't loaded, or if
-	 * it was, but if about:blank is also in the modified location */
-	if (embed->location_is_modified &&
-	    (strcmp (embed->location, "about:blank") ||
-	     (embed->modified_location &&
-	      !strcmp (embed->modified_location, "about:blank"))))
-	{
-		embed->location_is_modified = FALSE;
-		if (embed->modified_location)
-		{
-			g_free (embed->modified_location);
-			embed->modified_location = NULL;
-		}
-	}
-
-#ifdef ENABLE_NAUTILUS_VIEW
-	if (EMBED_IN_NAUTILUS (embed))
-	{
-		galeon_nautilus_view_set_location (embed->nautilus_view,
-						   embed->location);
-	}
-	else
-#endif
-	{
-		GaleonWindow *window;
-
-		/* get the parent window */
-		window = embed->parent_window;
-		return_if_not_window (window);
-		
-		/* update the text */
-		if (embed->is_active)
-			window_update_location_entry_text (window);
-	}
-}
-
 /**
  * embed_update_title: updates the embed's title.  also updates the tab
  * title and the parent window's title, if appropriate.
@@ -2195,8 +2120,7 @@ embed_move_to_existing_window (GaleonEmbed *embed, GaleonWindow *window,
 	if (orig_window == NULL)
 	{
 		window_add_embed (window, embed,
-				  EMBED_CREATE_FORCE_JUMP |
-				  EMBED_CREATE_RAISE_WINDOW);
+				  EMBED_CREATE_FORCE_JUMP);
 	}
 	/* simple case -- moving within the same window; just reorder
 	 * the page */
@@ -2457,22 +2381,19 @@ embed_save_modified_location (GaleonEmbed *embed)
 
 	entry_text = gtk_editable_get_chars 
 		(GTK_EDITABLE (window->location_entry), 0, -1);
-	if (entry_text && embed->location && 
-	    strcmp (entry_text, embed->location))
+	if (entry_text && (!embed->location ||
+	    (embed->location && strcmp (entry_text, embed->location))))
 	{
 		/* get location text */
 		embed->modified_location = entry_text;
-		embed->location_is_modified = TRUE;
 	}
 	else
 	{
 		/* otherwise, location isn't modified */
 		embed->modified_location = NULL;
-		embed->location_is_modified = FALSE;
 		if (entry_text) g_free (entry_text);
 	}
 	
-			
 	/* return if no selection was made */
 	if (!(GTK_EDITABLE (window->location_entry)->has_selection))
 		return;
@@ -2490,9 +2411,8 @@ embed_save_modified_location (GaleonEmbed *embed)
 		end = tmp;
 	}
 
-	text = gtk_editable_get_chars 
-		(GTK_EDITABLE (window->location_entry), 
-		 start, end);
+	text = gtk_editable_get_chars (GTK_EDITABLE (window->location_entry), 
+				       start, end);
 
 	/* free up stored per-window selection */
 	stored = gtk_object_get_data (GTK_OBJECT (embed->mozembed),
