@@ -628,7 +628,7 @@ galeon_nautilus_view_set_location (GaleonNautilusView *view,
 	/* don't inform nautilus about uris that it can't understand */
 	while (prefixes_to_ignore[i] != NULL)
 	{
-		if (strncmp (prefixes_to_ignore[i], uri, 
+		if (!strncmp (prefixes_to_ignore[i], uri, 
 			     strlen (prefixes_to_ignore[i])))
 			return;
 		i++;
