@@ -82,13 +82,22 @@ typedef enum
 						     embed not visible */
 	EMBED_CREATE_GRAB_FOCUS		= 1 << 6, /* grab the focus on the
 						     first page */
-	EMBED_CREATE_DONT_AUTOSAVE_INITIALLY = 1 << 7
-						  /* when set, the session
-						     will not be automatically
-						     backup up after the first
-						     location_change signal */
+	EMBED_CREATE_DONT_AUTOSAVE_SESSION = 1 << 7  /* set the embed's
+						     dont_autosave_session
+						     flag */
 } EmbedCreateFlags;
 
+/**
+ * DisplayType: Passed through to mozilla when creating a new view from the
+ * contents of an old one.
+ **/
+
+typedef enum
+{
+	DISPLAY_AS_SOURCE = 1U,
+	DISPLAY_NORMAL = 2U
+} DisplayType;
+
 /**
  * AutoreloadInfo: this structure contains information about when
  * and how the embed should be reloaded automatically
@@ -127,8 +136,7 @@ struct _GaleonEmbed
 	/* C++ helper, a "GaleonWrapper" */
         gpointer wrapper;
 
-	/* location and title.  modified_location is used to store a
-	 * modified location when the embed is not active. */
+	/* location and title */
 	gchar *location;
 	gchar *modified_location;
 	gchar *title;
@@ -169,17 +177,13 @@ struct _GaleonEmbed
 	/* secure page ? */
 	gboolean secure : 1;
 
-	/* if true, don't autosave the session file at location_changed
-	 * signal */
+	/* if true, don't autosave the session file
+	 * at the location_changed signal */
 	gboolean dont_autosave_session : 1;
 
 	/* has user stylesheet been applied ? */
 	gboolean usercss_applied : 1;
 
-	/* set whenever the location entry has been modified from the
-	 * actual location */
-	gboolean location_is_modified : 1;
-
 	/* creation flags */
 	EmbedCreateFlags create_flags;
 
@@ -210,8 +214,10 @@ struct _GaleonEmbed
 	gint gesture_release_signal;
 	guint gesture_button;
 
-	/* Used by the about:blank hack */
-	gchar *realurl;
+ 	/* used for scrolling */
+ 	gint scroll_motion_signal;
+ 	gint scroll_release_signal;
+ 	gint scroll_timeout_id;
 
 	/* Hack to work around javascript close called
 	   when we are already closing the window */
