@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000 Marco Pesenti Gritti
+ *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -31,9 +31,15 @@ void history_exit (void);
 void history_save (void);
 void history_clear (void);
 void history_visited (const gchar *url);
+void history_remove_item (const gchar *url);
 void history_set_page_title (const gchar *url, 
 			     gchar *title_locale,
 			     gchar *title_utf8);
+/**
+ * Warning: don't free or modify the returned string, because it points
+ * to internal history data. Can return NULL.
+ */
+const gchar *history_get_page_title (const gchar *url);
 gboolean history_is_visited (const gchar *url);
 const gchar *history_get_last_url (void);
 GList *history_get_host_list (void);
@@ -47,7 +53,6 @@ void history_show_dock (GaleonWindow *window);
 void history_resort_view (HistoryView *hv);
 void history_refresh_view (HistoryView *hv);
 void history_destroy_view (HistoryView *hv);
-void history_clear_recent_menu (void);
 void history_notifiers_remove (void);
 
 /* history_callbacks.c */
