@@ -101,15 +101,13 @@ static gboolean open_in_new_tab    = FALSE;  /* force open in a new tab?     */
 static gboolean noraise            = FALSE;  /* no raise                     */
 static gboolean open_in_new_window = FALSE;  /* force open in a new window?  */
 static gboolean open_fullscreen    = FALSE;  /* open galeon in full screen ? */
+static gboolean disable_crash_recovery = FALSE; /* disable crash recovery    */
 static gchar *session_filename     = NULL;   /* the session filename         */
 static gchar *geometry_string      = NULL;   /* the geometry string          */
 static gchar *bookmark_url         = NULL;   /* the temp bookmark to add     */
 static gboolean close_option       = FALSE;  /* --close                      */
 static gboolean quit_option        = FALSE;  /* --quit                       */
 
-/* mozilla version detected at runtime */
-int mozilla_version=0;
-
 /* command line argument parsing structure */
 static struct poptOption popt_options[] =
 {
@@ -133,6 +131,10 @@ static struct poptOption popt_options[] =
 	{ "load-session", 'l', POPT_ARG_STRING, &session_filename, 0,
 	  N_("Load the given session file"),
 	  N_("FILE") },
+	{ "disable-crash-recovery", 'd', POPT_ARG_NONE,
+	  &disable_crash_recovery, 0,
+	  N_("Disable Galeon's crash recovery dialog"),
+	  NULL },
 #ifdef ENABLE_APPLET
 	{ "panel", 'p', POPT_ARG_NONE, &galeon_panel_mode, 0,
 	  N_("Open as a panel applet"),
@@ -305,8 +307,7 @@ main (int argc, char *argv[])
 	for (i = 0; i < n_urls; i++)
 	{
 		last_embed = embed_create_after_embed (last_embed,
-			!tabbed_mode, url[i], EMBED_CREATE_RAISE_WINDOW |
-			EMBED_CREATE_FORCE_APPEND);
+			!tabbed_mode, url[i], EMBED_CREATE_FORCE_APPEND);
 		if (open_fullscreen && last_embed->parent_window)
 		{
 			window_set_fullscreen_mode (last_embed->parent_window, 
@@ -579,7 +580,7 @@ new_shell (CORBA_ORB orb)
 static gboolean
 galeon_init (int argc, char *argv[])
 {
-	static gchar *restart_argv[] = { "galeon", "--server", NULL };
+	static gchar *restart_argv[] = { "galeon", "--server", "--server-timeout", "0", NULL };
 	GnomeClient *session_client;
 	gboolean session_recovery;
 	gboolean vfs_success;
@@ -621,13 +622,16 @@ galeon_init (int argc, char *argv[])
 #endif
 
 	/* resume a crashed session, if any */
-	session_recovery = session_autoresume ();
+	if (!disable_crash_recovery) session_recovery = session_autoresume ();
+	else session_recovery = FALSE;
 
 	/* attach session-manager callbacks */
 	session_client = gnome_master_client ();
 	gnome_client_set_restart_command 
-		(session_client, galeon_server_mode ? 2 : 1, restart_argv);
+		(session_client, galeon_server_mode ? ( (galeon_server_timeout == 0) ? 4 : 2 ) : 1, restart_argv);
 	gnome_client_set_restart_style (GNOME_CLIENT (session_client),
+					(galeon_server_mode && (galeon_server_timeout == 0)) ?
+					GNOME_RESTART_IMMEDIATELY :
 					GNOME_RESTART_IF_RUNNING);
 	gtk_signal_connect (GTK_OBJECT (session_client), "save_yourself",
 			    (GtkSignalFunc)client_save_yourself_cb, NULL);
@@ -676,7 +680,6 @@ create_default_browser (void)
 	
 	/* build the default embed inside it */
 	embed = embed_create_in_window (window, NULL, NULL,
-					EMBED_CREATE_RAISE_WINDOW |
 					EMBED_CREATE_LOAD_DEFAULT_URL);
 	
 	/* show as fullscreen? */
@@ -750,6 +753,7 @@ galeon_nautilus_view_new (BonoboGenericFactory *factory, const char *id,
 void 
 galeon_exit (gboolean return_from_main_loop, gboolean unset_crashed_flag)
 {
+	GnomeClient *session_client;
 	extern GList *mozilla_notifiers;
 	
 	/**
@@ -788,6 +792,11 @@ galeon_exit (gboolean return_from_main_loop, gboolean unset_crashed_flag)
 	/* shut down GNOME subsystems */
 	gnome_vfs_shutdown ();
 
+	/* unregister at gnome session management */
+	session_client = gnome_master_client ();
+	gnome_client_set_restart_style (GNOME_CLIENT (session_client),
+					GNOME_RESTART_NEVER);
+
 	/* remove notifiers for mozilla prefs */
 	galeon_notification_remove (&mozilla_notifiers);
 
