@@ -116,10 +116,15 @@ GnomeUIInfo edit_menu_uiinfo[] =
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
 	  GDK_F, GDK_CONTROL_MASK, NULL },
 	{ GNOME_APP_UI_ITEM, N_("Find _next"),
-	  N_("Search for the next occurance of a word or phrase in the current page"),
+	  N_("Search for the next occurrence of a word or phrase in the current page"),
 	  window_menu_find_next_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
 	  GDK_G, GDK_CONTROL_MASK, NULL },
+	{ GNOME_APP_UI_ITEM, N_("Find pre_vious"),
+	  N_("Search for the previous occurrence of a word or phrase in the current page"),
+	  window_menu_find_previous_activate_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
+	  GDK_G, GDK_SHIFT_MASK|GDK_CONTROL_MASK, NULL },
 	  
 	GNOMEUIINFO_SEPARATOR,
 
@@ -139,7 +144,9 @@ GnomeUIInfo zoom_menu_uiinfo[] =
  	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_8, GDK_CONTROL_MASK, NULL },
 	  /* xgettext:no-c-format */
-	{ GNOME_APP_UI_ITEM, N_("_100%"), N_("Reset zoom level to 100%"),
+	{ GNOME_APP_UI_ITEM, N_("_100%"),
+	  /* xgettext:no-c-format */
+	  N_("Reset zoom level to 100%"),
 	  window_menu_zoom_100_activate_cb, NULL, NULL,
  	  GNOME_APP_PIXMAP_NONE, NULL,
 	  GDK_9, GDK_CONTROL_MASK, NULL },
@@ -561,7 +568,7 @@ GnomeUIInfo help_menu_uiinfo[] =
 {
         { GNOME_APP_UI_ITEM, N_("Galeon _FAQ"),
           N_("Open the Galeon FAQ (Frequently Asked Questions)"),
-          NULL, NULL, NULL,
+          window_menu_help_faq_activate_cb, NULL, NULL,
           GNOME_APP_PIXMAP_NONE, NULL,
           0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("Galeon _manual"), 
@@ -571,17 +578,17 @@ GnomeUIInfo help_menu_uiinfo[] =
 	  0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("Galeon _home page"),
 	  N_("Go to the Galeon homepage"),
-	  NULL, NULL, NULL,
+	  window_menu_help_homepage_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_HOME,
 	  0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("_Report a bug"),
 	  N_("Report a bug in Galeon"),
-	  NULL, NULL, NULL,
+	  window_menu_help_bugreport_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	{ GNOME_APP_UI_ITEM, N_("About _plugins"),
 	  N_("Show information about the currently installed plugins"),
-	  NULL, NULL, NULL,
+	  window_menu_help_about_plugins_activate_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, NULL,
 	  0, 0, NULL },
 	GNOMEUIINFO_MENU_ABOUT_ITEM (window_menu_help_about_activate_cb, NULL),
