@@ -16,22 +16,30 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifndef __mime_h
+#define __mime_h
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-void mime_add(gchar *name, gchar *program, gchar *extensions,
-	      MimeAction action);
-void mime_delete(MimeItem *m);
-void mime_update(MimeItem *m, gchar *name, gchar *program, 
-		 gchar *extensions, MimeAction action);
-void mime_get_entry(gchar **name, gchar **program, gchar **extensions,
-		    MimeAction *action);
+void mime_init_defaults (void);
+MimeItem* mime_item_get (const gchar *mime_type);
 MimeAction mime_get_action (const gchar *mime_type);
 void mime_set_action (const gchar *mime_type, MimeAction action);
+void mime_item_create (const gchar *mime_type, MimeAction action,
+		       const gchar *helperApp, gboolean terminalHelper,
+		       gboolean urlHelper, gboolean alwaysUse);
 MimeItem *mime_item_clone (MimeItem *m);
 void mime_item_free (MimeItem *m);
 
+void mime_db_save (void);
+void mime_db_shutdown (void);
+void mime_db_load (void);
+
 #ifdef __cplusplus
 }
 #endif
+
+#endif /* __mime_h */
+
