@@ -89,11 +89,11 @@ misc_string_strip_uline_accel (const gchar *text)
 			/* __ in the string is equal to _ in the output 
 			 * so include the _ in the output, skip over the 
 			 * second _ and continue scanning. */
-			g_string_sprintfa (out, "%.*s", u - cur + 1, cur);
+			g_string_sprintfa (out, "%.*s", (int)(u - cur + 1), cur);
 			cur = u + 2;
 		} else {
 			/* copy everything before the _ and skip over it */
-			g_string_sprintfa (out, "%.*s", u - cur , cur);
+			g_string_sprintfa (out, "%.*s", (int)(u - cur) , cur);
 			cur = u + 1;
 			/* only one accel per string, so we are done now */
 			break;
@@ -127,7 +127,7 @@ misc_string_escape_uline_accel (const gchar *text)
 	{
 		/* All we need to do is double every _, so include the _ in 
 		 * the output, add another _, and continue scanning. */
-		g_string_sprintfa (out, "%.*s_", u - cur + 1, cur);
+		g_string_sprintfa (out, "%.*s_", (int)(u - cur + 1), cur);
 		cur = u + 1;
 	}
 	if (cur && *cur)
@@ -528,7 +528,7 @@ misc_string_unescape_hexed_string (const gchar *text)
 	{
 		decodebuf[0]=u[1];
 		decodebuf[1]=u[2];
-		g_string_sprintfa (out, "%.*s%c", u-cur, cur, 
+		g_string_sprintfa (out, "%.*s%c", (int)(u-cur), cur, 
 					(char)strtol(decodebuf, NULL, 16));
 		cur = u + 3;
 	}
