@@ -37,8 +37,10 @@
 #include "stroke.h"
 #include "gestures.h"
 
+#include <sys/time.h>
 #include <time.h>
 #include <string.h>
+#include <math.h>
 #include <libgnomeui/gnome-app.h>
 #include <libgnomeui/gnome-app-helper.h>
 #include <libgnome/gnome-config.h>
@@ -46,8 +48,20 @@
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
 #include <gdk/gdkkeysyms.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 WrapperMouseEventInfo *mouse_down_info = NULL;
+
+/* used for autoscroll */
+GtkWidget *autoscroll_icon = NULL;
+guint scroll_start_x;
+guint scroll_start_y;
+gfloat scroll_step_x;
+gfloat scroll_step_y;
+gfloat scroll_roundoff_error_x;
+gfloat scroll_roundoff_error_y;
+gint scroll_msecs = 33;
+
 static void mozembed_gesture_connect (GaleonEmbed *embed,
 				      guint button);
 static gboolean mozembed_gesture_motion_cb (GtkWidget *widget,
@@ -59,6 +73,14 @@ static gboolean mozembed_gesture_press_cb (GtkWidget *widget,
 static gboolean mozembed_gesture_release_cb (GtkWidget *widget,
 					     GdkEventButton *e,
 					     GaleonEmbed *embed);
+static void mozembed_scroll_connect (GaleonEmbed *embed, guint x, guint y);
+static gboolean mozembed_scroll_motion_cb (GtkWidget *widget,
+					   GdkEventMotion *e,
+					   GaleonEmbed *embed);
+static gboolean mozembed_scroll_release_cb (GtkWidget *widget,
+					    GdkEventButton *e,
+					    GaleonEmbed *embed);
+static gint mozembed_scroll_timeout_cb (gpointer data);
 
 /**
  * mozembed_new_window_cb: GTKMOZEMBED SIGNAL, emitted any time a new 
@@ -98,7 +120,6 @@ mozembed_new_window_cb (GtkMozEmbed *dummy, GtkMozEmbed **retval,
 	/* create a new browser */
 	new_embed = embed_create_after_embed_chrome (embed, !open_in_tab,
 						     NULL,
-						     EMBED_CREATE_RAISE_WINDOW |
 						     EMBED_CREATE_DONT_SHOW,
 						     chrome_mask);
 
@@ -128,7 +149,6 @@ void new_window_orphan_cb (GtkMozEmbedSingle *embed,
 
 	/* create a new browser */
 	new_embed = embed_create_in_window_chrome (NULL, NULL, NULL,
-						   EMBED_CREATE_RAISE_WINDOW |
 						   EMBED_CREATE_DONT_SHOW,
 						   chrome_mask);
 
@@ -161,18 +181,11 @@ mozembed_visibility_cb (GtkMozEmbed *dummy, gboolean visibility,
 	/* set visiblity of this embed */
 	embed_set_visibility (embed, visibility);
 
-	if (visibility && 
-	    window->is_popup &&
-	    !window->layered)
+	/* set layer for popups */
+	if (visibility && window->is_popup && !window->layered)
 	{
 	        window_set_layer (window);
 	}
-
-	/* do embed init if necessary */
-	if (visibility && !embed->wrapper)
-	{
-		embed_wrapper_init (embed);
-	}
 }
 
 /**
@@ -198,34 +211,67 @@ void
 mozembed_location_changed_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 {
 	GaleonWindow *window;
+	gchar *new_location;
 	gint zoom;
 
  	return_if_not_sane_embed (embed);
 	window = embed->parent_window;
 
-	/* update in embed (and window if necessary) */
-	embed_update_location (embed);
-
-	/* update the buttons view */
-	if (embed->is_active && window)
-		window_update_nav_controls (window);
-
-	/* autosave the updated session, if about:blank hasn't been loaded */
-	if (strcmp ("about:blank", embed->location))
+#ifdef ENABLE_NAUTILUS_VIEW
+	if (!EMBED_IN_NAUTILUS (embed))
+#endif
 	{
-		/* make sure we're not recovering from a crash (in which
-		 * case, we don't want to autosave the session yet) */
-		if (!embed->dont_autosave_session)
-			session_autosave ();
-		else embed->dont_autosave_session = FALSE;
+		return_if_not_window (window);
 	}
 
+	/* get the new location */
+	new_location = mozilla_get_real_url (embed);
+
+	/* clear modified_location, assuming that we haven't just 
+	 * failed a page load in a newly-created embed */
+	if (embed->modified_location && (embed->location ||
+	    strcmp (new_location, "about:blank")))
+	{
+		g_free (embed->modified_location);
+		embed->modified_location = NULL;
+	}
+
+	/* update location in embed */
+	if (embed->location) g_free (embed->location);
+	embed->location = new_location;
+
+	/* update the window as appropriate */
+#ifdef ENABLE_NAUTILUS_VIEW
+	if (EMBED_IN_NAUTILUS (embed))
+	{
+		galeon_nautilus_view_set_location (embed->nautilus_view,
+						   embed->location);
+	}
+	else
+#endif
+	{
+		if (embed->is_active)
+		{
+			window_update_nav_controls (window);
+			window_update_location_entry_text (window);
+		}
+	}
+
+	/* save the session, if we haven't been told not to */
+	if (!embed->dont_autosave_session)
+	{
+		session_autosave ();
+	}
+	/* we'll save next time */
+	else embed->dont_autosave_session = FALSE;
+
 	/* check zoom setting for the new site, unless it has 
 	   a javascript URI */
 	if (embed->location 
 	    && strncmp ("javascript:", embed->location, 11))
 	{
 		zoom = history_get_zoom (embed->location);
+
 		if (zoom == 0 && embed->zoom_auto_set)
 		{
 			embed_set_zoom (embed, 100, FALSE);
@@ -235,8 +281,11 @@ mozembed_location_changed_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 			embed_set_zoom (embed, zoom, FALSE);
 			embed->zoom_auto_set = TRUE;
 		}
+
 		/* update the spinner */
-		if (window != NULL)
+#ifdef ENABLE_NAUTILUS_VIEW
+		if (!EMBED_IN_NAUTILUS (embed))
+#endif
 		{
 			window_update_zoom (window);
 		}
@@ -269,10 +318,13 @@ mozembed_load_started_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	return_if_not_sane_embed (embed);
 	window = embed->parent_window;
 
-	/* initialise embed whenever a document is first loaded */
-	if (embed->wrapper == NULL)
-		embed_wrapper_init (embed);
-	
+#ifdef ENABLE_NAUTILUS_VIEW
+	if (!EMBED_IN_NAUTILUS (embed))
+#endif
+	{
+		return_if_not_window (window);
+	}
+
 	embed->load_started++;
 
 	/* if we haven't already, start the window spinner, set the
@@ -283,29 +335,37 @@ mozembed_load_started_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 		embed_progress_clear (embed);
 		embed->when_started = time (NULL);
 
-		/* start spinner if this is the active embed */
-		if (embed->is_active && window)
-			spinner_start (window);
 #if ENABLE_NAUTILUS_VIEW
 		if (EMBED_IN_NAUTILUS (embed))
+		{
 			galeon_nautilus_view_report_load_underway 
 				(embed->nautilus_view);
+		}
+		else
 #endif
+		{
+			if (embed->is_active)
+				spinner_start (window);
+		}
 	}
 
 	/* update the buttons view */
-	if (embed->is_active && window)
+#if ENABLE_NAUTILUS_VIEW
+	if (!EMBED_IN_NAUTILUS (embed))
+#endif
 	{
-		window_update_nav_controls (window);
-		window_statusbar_update_message (window);
-		window_statusbar_update_progress_bar (window);
-		if (window->csslist != NULL)
-			gtk_widget_set_sensitive (window->csslist, FALSE);
+		if (embed->is_active)
+		{
+			window_update_nav_controls (window);
+			window_statusbar_update_message (window);
+			window_statusbar_update_progress_bar (window);
+			if (window->csslist != NULL)
+				gtk_widget_set_sensitive (
+						window->csslist, FALSE);
+			favicon_update_drag_handle (embed, FALSE);
+		}
 	}
 
-	/* update drag pixmap */
-	favicon_update_drag_handle (embed, FALSE);
-
 	/* no user sheet has been applied yet */
 	embed->usercss_applied = FALSE;
 }
@@ -328,28 +388,37 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 	embed->load_started--;
 	embed_progress_clear (embed);
 
-	if (embed->load_started == 0)
-	{
-		/* stop spinner if this is the active embed */
-		if (embed->is_active && window)
-			spinner_stop (window);
-#if ENABLE_NAUTILUS_VIEW
-		if (EMBED_IN_NAUTILUS (embed))
-			galeon_nautilus_view_report_load_complete
-				(embed->nautilus_view);
+	/* return if there are still loads going on */
+	if (embed->load_started) return;
+
+#ifdef ENABLE_NAUTILUS_VIEW
+	if (!EMBED_IN_NAUTILUS (embed))
 #endif
+	{
+		return_if_not_window (window);
+	}
 
-		/* new content, so only viewed if this is already active */
+	if (!embed->location)
+	{
+		gtk_moz_embed_load_url (GTK_MOZ_EMBED (embed->mozembed),
+					"about:blank");
+	}
+
+#if ENABLE_NAUTILUS_VIEW
+	if (EMBED_IN_NAUTILUS (embed))
+	{
+		galeon_nautilus_view_report_load_complete
+			(embed->nautilus_view);
+	}
+	else
+#endif
+	{
+		if (embed->is_active) spinner_stop (window);
+
+		/* new content, so only viewed if
+		 * this is already active */
 		embed->has_been_viewed = embed->is_active;
-
-		/* update tab status */
 		embed_update_tab_status (embed);
-		
-		/* load saved alternate stylesheet */
-		stylesheets_load_authorsheet (embed);
-
-		/* load user stylesheet */
-		stylesheets_load_usersheet (embed);
 
 		/* update drag pixmap */
 		realurl = mozilla_get_real_url (embed);
@@ -359,56 +428,32 @@ mozembed_load_finished_cb (GtkMozEmbed *dummy, GaleonEmbed *embed)
 		g_free (realurl);
 
 		/* Update the recent menu */
-		if (window && embed->location)
+		if (embed->location)
 		{
-			window_add_recent_menu_entry
-				(window, 
+			window_add_recent_menu_entry (window, 
 				 embed->location, embed->title);
 		}
 	}
 
-	/* update the buttons view */
-	if (embed->is_active)
+	/* load saved alternate stylesheet */
+	stylesheets_load_authorsheet (embed);
+
+	/* load user stylesheet */
+	stylesheets_load_usersheet (embed);
+
+	/* update window items */
+#ifdef ENABLE_NAUTILUS_VIEW
+	if (!EMBED_IN_NAUTILUS (embed))
+#endif
 	{
-		if (window)
+		if (embed->is_active)
 		{
 			window_update_nav_controls (window);
 			window_statusbar_update_message (window);
 			window_statusbar_update_progress_bar (window);
-		}
-
-#ifdef ENABLE_NAUTILUS_VIEW
-		if (!EMBED_IN_NAUTILUS (embed))
-		{
-#endif
-			/* check if the wrapper is already initialized */
 			stylesheets_menu_build (embed);
 			link_interfaces_set_sensitivity (embed);
-#ifdef ENABLE_NAUTILUS_VIEW
 		}
-#endif
-	}
-
-	/* hack to workaround a bunch of problems when no document is loaded */
-	if (embed->realurl != NULL)
-	{
-		/* Nullify before loading the url,
-		   I dont get how that happen but the finish callback appear to be
-		   called again before finish_cb exit */
-		gchar *tmp = g_strdup (embed->realurl);
-		g_free (embed->realurl);
-		embed->realurl = NULL;
-
-		embed_load_url (embed, tmp);
-
-		if ((embed->create_flags & EMBED_CREATE_GRAB_FOCUS) &&
-		    (window->visible_embeds == 1 || 
-		     eel_gconf_get_boolean (CONF_TABS_TABBED_AUTOJUMP)))
-		{
-			embed_grab_focus (embed);
-		}
-
-		g_free (tmp);
 	}
 }
 
@@ -425,32 +470,16 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy,
 	return_if_not_sane_embed (embed);
 	window = embed->parent_window;
 
-	/* if this is a new embed, set modified_location, so the location
-	 * will show up in the entry */
-	if (!embed->location && !embed->modified_location && uri &&
-	    strcmp (uri, "about:blank") &&
-	    strcmp (uri, "about:parser-dummy-request") &&
-	    strcmp (uri, "about:layout-dummy-request"))
+	if (!embed->wrapper)
 	{
-		embed->modified_location = g_strdup (uri);
-		embed->location_is_modified = TRUE;
+		embed_wrapper_init (embed);
+	}
 
 #ifdef ENABLE_NAUTILUS_VIEW
-		if (EMBED_IN_NAUTILUS (embed))
-		{
-			galeon_nautilus_view_set_location (
-						embed->nautilus_view,
-						embed->modified_location);
-		}
-		else
+	if (!EMBED_IN_NAUTILUS (embed))
 #endif
-		{
-			return_if_not_window (window);
-			
-			/* update the text */
-			if (embed->is_active)
-				window_update_location_entry_text (window);
-		}
+	{
+		return_if_not_window (window);
 	}
 
 	/* clear temporary message, if any */
@@ -488,6 +517,30 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy,
 		if (flags & GTK_MOZ_EMBED_FLAG_START)
 		{
 			embed->load_status_message = _("Loading site...");
+
+			/* store url in modified_location if no url has
+			 * been loaded yet, and if we aren't loading
+			 * about:blank because the real load failed */
+			if (!embed->location && uri &&
+			    strcmp (uri, "about:blank"))
+			{
+				embed->modified_location = g_strdup (uri);
+
+				/* update the location entry */
+#ifdef ENABLE_NAUTILUS_VIEW
+				if (!EMBED_IN_NAUTILUS (embed))
+#endif
+				{
+					if (embed->is_active)
+					{
+					window_update_location_entry_text (
+								window);
+					}
+				}
+
+				/* autosave the session */
+				session_autosave ();
+			}
 		}
 		else if (flags & GTK_MOZ_EMBED_FLAG_STOP)
 		{
@@ -522,6 +575,8 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy,
  */
 		break;
 	}
+
+	/* display the update */
 #ifdef ENABLE_NAUTILUS_VIEW
 	if (EMBED_IN_NAUTILUS (embed))
 	{
@@ -542,13 +597,14 @@ void mozembed_net_status_change_cb (GtkMozEmbed *dummy,
 				 embed->load_percent);
 		}
 	}
+	else
 #endif
-
-	/* display the update */
-	if (embed->is_active && window)
 	{
-		window_statusbar_update_message (window);
-		window_statusbar_update_progress_bar (window);
+		if (embed->is_active)
+		{
+			window_statusbar_update_message (window);
+			window_statusbar_update_progress_bar (window);
+		}
 	}
 	// NAUTIFIXME: report the failed load
 }
@@ -705,6 +761,10 @@ mozembed_dom_mouse_down_cb (GtkMozEmbed *dummy, gpointer dom_event,
 				/* start the gesture */
 				mozembed_gesture_connect (embed, 2);
 				break;
+ 			case 4:
+ 				/* scroll */
+				/* we do this in the click event, not here */
+ 				break;
 			default:
 				/* handled elsewhere */
 				break;
@@ -836,6 +896,18 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 					/* we do this on the mouse_down
 					 * event, not here */
 					break;
+				/* scroll */
+				case 4:
+					/* make sure we aren't currently in
+					 * a scroll */
+ 					if (embed->scroll_release_signal)
+						break;
+ 
+					/* start the scrolling */
+					mozembed_scroll_connect (embed,
+						info->x_root, info->y_root);
+
+					break;
 				/* do nothing */
 				default:
 					break;
@@ -847,6 +919,7 @@ mozembed_dom_mouse_click_cb (GtkMozEmbed *dummy, gpointer dom_event,
 
 	case 0: 
 		if ((info->ctx.context & CONTEXT_IMAGE) &&
+		    !(info->ctx.context & CONTEXT_INPUT) &&
 		    (info->modifier & CTRL_KEY))
 		{
 			embed_save_image (embed, info->ctx.img, FALSE);
@@ -1105,7 +1178,7 @@ mozembed_security_change_cb (GtkMozEmbed *dummy, gpointer request,
 			     guint state, GaleonEmbed *embed)
 {	
 	GaleonWindow *window;
-	gboolean security_state = (state!=0);
+	gboolean security_state = (state & 2) >> 1;
 
 	/* get parent window */
 	return_if_not_sane_embed (embed);
@@ -1141,8 +1214,9 @@ mozembed_security_change_cb (GtkMozEmbed *dummy, gpointer request,
 					(_("Security level: %s"), level);
 			}
 			
-			window_statusbar_set_security_icon 
-				(window, security_state, tooltip);
+			if (embed->is_active)
+				window_statusbar_set_security_icon 
+					(window, security_state, tooltip);
 			embed->security_tooltip = tooltip;
 			g_free (tmp);
 			g_free (level);
@@ -1206,7 +1280,7 @@ mozembed_destroy_cb (GtkObject *object, GaleonEmbed *embed)
 		mozilla_wrapper_destroy (embed);
 	}
 
-	/* remove from list of embeds */
+	/* remove from global list of embeds */
 	all_embeds = g_list_remove (all_embeds, embed);
 
 	/* from list of embeds in parent */
@@ -1219,7 +1293,7 @@ mozembed_destroy_cb (GtkObject *object, GaleonEmbed *embed)
 		window_close (window);
 	}
 	/* window_close will autosave the updated session, but if it isn't
-	 * called, we should save ourselves */
+	 * called, we need to do the autosave ourselves */
 	else
 	{
 		session_autosave ();
@@ -1432,16 +1506,14 @@ mozembed_gesture_release_cb (GtkWidget *widget, GdkEventButton *e,
 				{
 					new = embed_create_after_embed (embed,
 						TRUE,
-						mouse_down_info->ctx.link,
-						EMBED_CREATE_RAISE_WINDOW);
+						mouse_down_info->ctx.link, 0);
 				}
 				else
 				{
 					new = embed_create_after_embed (embed,
 						TRUE, NULL,
 						EMBED_CREATE_LOAD_DEFAULT_URL |
-						EMBED_CREATE_FORCE_JUMP |
-						EMBED_CREATE_RAISE_WINDOW);
+						EMBED_CREATE_FORCE_JUMP);
 				}
 				g_assert (new != NULL);
 				window_grab_location_focus
@@ -1457,16 +1529,14 @@ mozembed_gesture_release_cb (GtkWidget *widget, GdkEventButton *e,
 				{
 					new = embed_create_after_embed (embed,
 						FALSE,
-						mouse_down_info->ctx.link,
-						EMBED_CREATE_RAISE_WINDOW);
+						mouse_down_info->ctx.link, 0);
 				}
 				else
 				{
 					new = embed_create_after_embed (embed,
 						FALSE, NULL,
 						EMBED_CREATE_LOAD_DEFAULT_URL |
-						EMBED_CREATE_FORCE_JUMP |
-						EMBED_CREATE_RAISE_WINDOW);
+						EMBED_CREATE_FORCE_JUMP);
 				}
 				g_assert (new != NULL);
 				window_grab_location_focus
@@ -1494,13 +1564,11 @@ mozembed_gesture_release_cb (GtkWidget *widget, GdkEventButton *e,
 				break;
 			case CLONE_WINDOW:
 				embed_create_after_embed (embed,
-					TRUE, embed->location,
-					EMBED_CREATE_RAISE_WINDOW);
+					TRUE, embed->location, 0);
 				break;
 			case CLONE_TAB:
 				embed_create_after_embed (embed,
-					FALSE, embed->location,
-					EMBED_CREATE_RAISE_WINDOW);
+					FALSE, embed->location, 0);
 				break;
 			case UP:
 				embed_go_up (embed, 0, 0);
@@ -1622,3 +1690,249 @@ mozembed_gesture_release_cb (GtkWidget *widget, GdkEventButton *e,
 
 	return TRUE;
 }
+
+/**
+ * mozembed_scroll_connect: connects scrolling-related signals and grabs
+ * the pointer
+ */
+static void
+mozembed_scroll_connect (GaleonEmbed *embed, guint x, guint y)
+{
+	static GdkCursor *cursor = NULL;
+	GtkWidget *mozembed;
+
+	return_if_not_embed (embed);
+	mozembed = GTK_WIDGET (embed->mozembed);
+
+	/* get a new cursor, if necessary */
+	if (!cursor) cursor = gdk_cursor_new (GDK_FLEUR);
+
+	/* initialize the autoscroll icon, if necessary */
+	if (!autoscroll_icon)
+	{
+		gchar *filename;
+		GdkPixbuf *pixbuf;
+		GdkPixmap *pixmap;
+		GdkBitmap *bitmap;
+		GtkWidget *pix;
+
+		filename = g_concat_dir_and_file (SHARE_DIR,
+						  "autoscroll.xpm");
+		pixbuf = gdk_pixbuf_new_from_file (filename);
+		g_free (filename);
+		g_return_if_fail (pixbuf);
+		gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap,
+						   &bitmap, 128);
+		gdk_pixbuf_unref (pixbuf);
+
+		gtk_widget_push_visual (gdk_rgb_get_visual ());
+		gtk_widget_push_colormap (gdk_rgb_get_cmap ());
+		autoscroll_icon = gtk_window_new (GTK_WINDOW_POPUP);
+		pix = gtk_pixmap_new (pixmap, bitmap);
+		gtk_widget_realize (autoscroll_icon);
+		gtk_container_add (GTK_CONTAINER (autoscroll_icon), pix);
+		gtk_widget_shape_combine_mask (autoscroll_icon, bitmap, 0, 0);
+		gtk_widget_pop_visual ();
+		gtk_widget_pop_colormap ();
+		
+		gdk_pixmap_unref (pixmap);
+		gdk_bitmap_unref (bitmap);
+
+		gtk_widget_show_all (pix);
+	}
+
+	/* show icon */
+	gtk_widget_set_uposition (autoscroll_icon, x - 12, y - 12);
+	gtk_widget_show (autoscroll_icon);
+
+	/* set positions */
+	scroll_start_x = x;
+	scroll_start_y = y;
+	scroll_step_x = 0;
+	scroll_step_y = 0;
+	scroll_roundoff_error_x = 0;
+	scroll_roundoff_error_y = 0;
+
+	/* attach signals */
+	embed->scroll_motion_signal =
+		gtk_signal_connect (GTK_OBJECT (mozembed),
+			"motion_notify_event",
+			GTK_SIGNAL_FUNC (mozembed_scroll_motion_cb), embed);
+	embed->scroll_release_signal =
+		gtk_signal_connect (GTK_OBJECT (mozembed),
+			"button_release_event",
+			GTK_SIGNAL_FUNC (mozembed_scroll_release_cb), embed);
+	embed->scroll_timeout_id =
+		gtk_timeout_add (scroll_msecs,
+				 mozembed_scroll_timeout_cb, embed);
+
+	/* grab the pointer */
+	gtk_grab_add (mozembed);
+	gdk_pointer_grab (mozembed->window, FALSE,
+			  GDK_POINTER_MOTION_MASK |
+			  GDK_BUTTON_RELEASE_MASK,
+			  NULL, cursor, GDK_CURRENT_TIME);
+}
+
+/**
+ * mozembed_scroll_disconnect: disconnects scrolling-related signals and
+ * ungrabs the pointer
+ */
+void
+mozembed_scroll_disconnect (GaleonEmbed *embed)
+{
+	GtkWidget *mozembed;
+
+	return_if_not_embed (embed);
+	mozembed = GTK_WIDGET (embed->mozembed);
+
+	/* ungrab the pointer if it's grabbed */
+	if (gdk_pointer_is_grabbed ())
+	{
+		gdk_pointer_ungrab (GDK_CURRENT_TIME);
+		gtk_grab_remove (mozembed);
+	}
+
+	/* hide the icon */
+	gtk_widget_hide (autoscroll_icon);
+
+	/* disconnect all of the signals */
+	if (embed->scroll_motion_signal)
+	{
+		gtk_signal_disconnect (GTK_OBJECT (mozembed),
+				       embed->scroll_motion_signal);
+		embed->scroll_motion_signal = 0;
+	}
+	if (embed->scroll_release_signal)
+	{
+		gtk_signal_disconnect (GTK_OBJECT (mozembed),
+				       embed->scroll_release_signal);
+		embed->scroll_release_signal = 0;
+	}
+	if (embed->scroll_timeout_id)
+	{
+		gtk_timeout_remove (embed->scroll_timeout_id);
+		embed->scroll_timeout_id = 0;
+	}
+}
+
+/**
+ * mozembed_scroll_motion_cb: handles scrolling-related motion events in
+ * the mozembed
+ */
+static gboolean
+mozembed_scroll_motion_cb (GtkWidget *widget, GdkEventMotion *e,
+			   GaleonEmbed *embed)
+{
+	gint x_dist, x_dist_abs, y_dist, y_dist_abs;
+
+	return_val_if_not_embed (embed, TRUE);
+
+	/* get distance between scroll center and cursor */
+	x_dist = e->x_root - scroll_start_x;
+	x_dist_abs = abs (x_dist);
+	y_dist = e->y_root - scroll_start_y;
+	y_dist_abs = abs (y_dist);
+
+	/* calculate scroll step */
+	if (y_dist_abs <= 48.0)
+		scroll_step_y = (float) (y_dist / 4) / 6.0;
+	else if (y_dist > 0)
+		scroll_step_y = (y_dist - 48.0) / 2.0 + 2.0;
+	else scroll_step_y = (y_dist + 48.0) / 2.0 - 2.0;
+
+	if (x_dist_abs <= 48.0)
+		scroll_step_x = (float) (x_dist / 4) / 6.0;
+	else if (x_dist > 0)
+		scroll_step_x = (x_dist - 48.0) / 2.0 + 2.0;
+	else scroll_step_x = (x_dist + 48.0) / 2.0 - 2.0;
+
+	return TRUE;
+}
+
+/**
+ * mozembed_scroll_release_cb: ends the scrolling
+ */
+static gboolean
+mozembed_scroll_release_cb (GtkWidget *widget, GdkEventButton *e,
+			    GaleonEmbed *embed)
+{
+	if (e->type != GDK_BUTTON_RELEASE)
+		return FALSE;
+
+	return_val_if_not_embed (embed, TRUE);
+
+	/* ungrab and disconnect */
+	mozembed_scroll_disconnect (embed);
+
+	return TRUE;
+}
+
+/**
+ * mozembed_scroll_timeout_cb: timeout that does the scrolling
+ */
+static gint
+mozembed_scroll_timeout_cb (gpointer data)
+{
+	struct timeval start_time, finish_time;
+	long elapsed_msecs;
+	GaleonEmbed *embed = (GaleonEmbed *) data;
+	gfloat scroll_step_y_adj;
+	gint scroll_step_y_int;
+
+	return_val_if_not_embed (embed, TRUE);
+
+	/* return if we're not supposed to scroll */
+	if (!scroll_step_y) return TRUE;
+
+	/* calculate the number of pixels to scroll */
+	scroll_step_y_adj = scroll_step_y * scroll_msecs / 33;
+	scroll_step_y_int = scroll_step_y_adj;
+	scroll_roundoff_error_y += (scroll_step_y_adj - scroll_step_y_int);
+
+	if (fabs (scroll_roundoff_error_y) >= 1.0)
+	{
+		scroll_step_y_int += scroll_roundoff_error_y;
+		scroll_roundoff_error_y -= (gint) scroll_roundoff_error_y;
+	}
+
+	/* exit if we're not supposed to scroll yet */
+	if (!scroll_step_y_int) return TRUE;
+
+	/* get the time before we tell mozilla to scroll */
+	gettimeofday (&start_time, NULL);
+
+	/* do scrolling, moving at a constart speed regardless of the
+	 * scrolling delay */
+	/* FIXME: if mozilla is able to do diagonal scrolling in a
+	 * reasonable manner at some point, this should be changed to
+	 * calculate x and pass both values instead of just y */
+	mozilla_scroll_by (embed, 0, scroll_step_y_int);
+
+	/* find out how long the scroll took */
+	gettimeofday (&finish_time, NULL);
+	elapsed_msecs = (1000000L * finish_time.tv_sec + finish_time.tv_usec -
+			 1000000L * start_time.tv_sec - start_time.tv_usec) /
+			1000;
+
+	/* check if we should update the scroll delay */
+	if ((elapsed_msecs >= scroll_msecs + 5) ||
+	    ((scroll_msecs > 20) && (elapsed_msecs < scroll_msecs - 10)))
+	{
+		/* update the scrolling delay, with a
+		 * minimum delay of 20 ms */
+		scroll_msecs =
+			(elapsed_msecs + 10 >= 20) ? elapsed_msecs + 10 : 20;
+
+		/* create new timeout with adjusted delay */
+		embed->scroll_timeout_id =
+			gtk_timeout_add (scroll_msecs,
+					 mozembed_scroll_timeout_cb, embed);
+
+		/* kill the old timeout */
+		return FALSE;
+	}
+
+	/* don't kill timeout */
+	return TRUE;
+}
