@@ -147,7 +147,7 @@ GFilePicker::GFilePicker (PRBool showContentCheck, FileFormat *fileFormats) : mS
 	mFileFormats = fileFormats;
 	mFile = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
 	mDisplayDirectory = do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-	mDisplayDirectory->InitWithPath(g_get_home_dir());
+	mDisplayDirectory->InitWithNativePath(nsDependentCString(g_get_home_dir()));
 }
 
 GFilePicker::~GFilePicker()
@@ -218,7 +218,6 @@ NS_IMETHODIMP GFilePicker::SetDefaultString(const PRUnichar * aDefaultString)
 	return NS_OK;
 }
 
-#if MOZILLA_VERSION>VERSION3(0,9,8)
 /* attribute wstring defaultExtension; */
 // Again, due to the crippled file selector, we can't really
 // do anything here.
@@ -226,11 +225,11 @@ NS_IMETHODIMP GFilePicker::GetDefaultExtension(PRUnichar * *aDefaultExtension)
 {
     return NS_ERROR_NOT_IMPLEMENTED;
 }
+
 NS_IMETHODIMP GFilePicker::SetDefaultExtension(const PRUnichar * aDefaultExtension)
 {
     return NS_OK;
 }
-#endif
 
 /* attribute nsILocalFile displayDirectory; */
 NS_IMETHODIMP GFilePicker::GetDisplayDirectory(nsILocalFile * *aDisplayDirectory)
@@ -279,16 +278,15 @@ NS_IMETHODIMP GFilePicker::Show(PRInt16 *_retval)
 		cFileName = mozilla_unicode_to_locale (mDefaultString.get());
 	}
 
-	char *cDirName;
-	mDisplayDirectory->GetPath (&cDirName);
+	nsCAutoString cDirName;
+	mDisplayDirectory->GetNativePath (cDirName);
 
-	char *cFullPath = g_strconcat (cDirName, "/", cFileName, NULL);
+	nsCAutoString cFullPath = cDirName + NS_LITERAL_CSTRING("/") +
+				  nsDependentCString(cFileName ? cFileName : "");
 	g_free (cFileName);
-	nsMemory::Free (cDirName);
 
 	gtk_file_selection_set_filename (GTK_FILE_SELECTION(mFileSelector),
-				 	 cFullPath);
-	g_free (cFullPath);
+				 	 cFullPath.get());
 
 	if (!mFilter.IsEmpty())
 	{
@@ -545,11 +543,12 @@ filePicker_save_as_ok_cb (GtkButton *button, GFilePicker *aFilePicker)
 		}
 	}
 
-	aFilePicker->mFile->InitWithPath (aFileName);
+	aFilePicker->mFile->InitWithNativePath (nsDependentCString(aFileName));
 
 	if (aFilePicker->mMode == nsIFilePicker::modeGetFolder)
 	{
-		aFilePicker->mDisplayDirectory->InitWithPath (aFileName);
+		aFilePicker->mDisplayDirectory->
+			InitWithNativePath (nsDependentCString(aFileName));
 		aFilePicker->mDefaultString = NS_LITERAL_STRING("");
 	}
 	else
@@ -557,9 +556,7 @@ filePicker_save_as_ok_cb (GtkButton *button, GFilePicker *aFilePicker)
 		nsCOMPtr<nsIFile> aDirectory;
 		aFilePicker->mFile->GetParent (getter_AddRefs(aDirectory));
 		aFilePicker->mDisplayDirectory = do_QueryInterface (aDirectory);
-		nsXPIDLString leafName;
-		aFilePicker->mFile->GetUnicodeLeafName (getter_Copies(leafName));
-		aFilePicker->mDefaultString = leafName;
+		aFilePicker->mFile->GetLeafName (aFilePicker->mDefaultString);
 	}
 
 	g_free (aFileName);
@@ -673,7 +670,7 @@ show_file_picker (GtkWidget *parentWidget, const char* title,
 
 	nsCOMPtr<nsILocalFile> aDir = 
 				do_CreateInstance (NS_LOCAL_FILE_CONTRACTID);
-	aDir->InitWithPath (expanded_directory);
+	aDir->InitWithNativePath (nsDependentCString(expanded_directory));
 	g_free (expanded_directory);
 
 	aFilePicker->InitWithGtkWidget (parentWidget, title, mode);
@@ -706,10 +703,9 @@ show_file_picker (GtkWidget *parentWidget, const char* title,
 			g_free (*ret_fullpath);
 		nsCOMPtr<nsILocalFile> aFile;
 		aFilePicker->GetFile (getter_AddRefs(aFile));
-		char *tempFullPathStr;
-		aFile->GetPath (&tempFullPathStr);
-		*ret_fullpath = g_strdup (tempFullPathStr);
-		nsMemory::Free (tempFullPathStr);
+		nsCString tempFullPathStr;
+		aFile->GetNativePath (tempFullPathStr);
+		*ret_fullpath = g_strdup (tempFullPathStr.get());
 		delete aFilePicker;
 		return TRUE;
 	}
