@@ -16,8 +16,13 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <libgnome/gnome-help.h>
 
+#include "nsReadableUtils.h"
 #include "nsIServiceManager.h"
 #include "nsIIOService.h"
 #include "nsNetCID.h"
@@ -46,7 +51,7 @@ NS_IMETHODIMP GGHelpProtocolHandler::NewChannel(nsIURI *aURI,
 
 	mDocName = NS_LITERAL_CSTRING ("file:///"); 
 
-	if (strstr(path.get(), ".html"))
+	if (FindInReadable(NS_LITERAL_CSTRING(".html"), path))
 	{
 		mDocName += (host + path);
 	}
