@@ -61,6 +61,7 @@ const CharsetInfo charset_trans_array[] = {
 	{N_("Chinese Simplified (GB2312)"),       "GB2312",                LG_CHINESE},
 	{N_("Chinese Simplified (GBK)"),          "x-gbk",                 LG_CHINESE},
 	{N_("Chinese Simplified (HZ)"),           "HZ-GB-2312",	           LG_CHINESE},
+	{N_("Chinese Simplified (Windows-936)"),  "windows-936",           LG_CHINESE},
 	{N_("Chinese Traditional (Big5)"),        "Big5",                  LG_CHINESE},
 	{N_("Chinese Traditional (Big5-HKSCS)"),  "Big5-HKSCS",	           LG_CHINESE},
 	{N_("Chinese Traditional (EUC-TW)"),      "x-euc-tw",              LG_CHINESE},
@@ -76,6 +77,7 @@ const CharsetInfo charset_trans_array[] = {
 	{N_("Cyrillic/Ukrainian (MacUkrainian)"), "x-mac-ukrainian",       LG_CYRILLIC},
 	{N_("English (US-ASCII)"),                "us-ascii",              LG_WESTERN},
 	{N_("Farsi (MacFarsi)"),                  "x-mac-farsi",           LG_OTHER},
+	{N_("Georgian (GEOSTD8)"),                "geostd8",               LG_OTHER},
 	{N_("Greek (ISO-8859-7)"),                "ISO-8859-7",            LG_GREEK},
 	{N_("Greek (MacGreek)"),                  "x-mac-greek",           LG_GREEK},
 	{N_("Greek (Windows-1253)"),              "windows-1253",          LG_GREEK},
@@ -92,10 +94,12 @@ const CharsetInfo charset_trans_array[] = {
 	{N_("Japanese (ISO-2022-JP)"),            "ISO-2022-JP",           LG_JAPANESE},
 	{N_("Japanese (Shift_JIS)"),              "Shift_JIS",             LG_JAPANESE},
 	{N_("Korean (EUC-KR)"),                   "EUC-KR",                LG_KOREAN},
+	{N_("Korean (ISO-2022-KR)"),              "ISO-2022-KR",           LG_KOREAN},
 	{N_("Korean (JOHAB)"),                    "x-johab",               LG_KOREAN},
 	{N_("Korean (UHC)"),                      "x-windows-949",         LG_KOREAN},
 	{N_("Nordic (ISO-8859-10)"),              "ISO-8859-10",           LG_OTHER},
 	{N_("Romanian (MacRomanian)"),            "x-mac-romanian",        LG_OTHER},
+	{N_("Romanian (ISO-8859-16)"),            "ISO-8859-16",           LG_OTHER},
 	{N_("South European (ISO-8859-3)"),       "ISO-8859-3",            LG_OTHER},
 	{N_("Thai (TIS-620)"),                    "TIS-620",               LG_OTHER},
 	{N_("Turkish (IBM-857)"),                 "IBM857",                LG_TURKISH},
@@ -122,8 +126,7 @@ const CharsetInfo charset_trans_array[] = {
 	/* charsets whithout posibly translatable names */
 	{"T61.8bit",                              "T61.8bit",              LG_OTHER},
 	{"x-imap4-modified-utf7",                 "x-imap4-modified-utf7", LG_UNICODE},
-	{"x-u-escaped",                           "x-u-escaped",           LG_OTHER},
-	{"windows-936",                           "windows-936",           LG_OTHER}
+	{"x-u-escaped",                           "x-u-escaped",           LG_OTHER}
 };
 
 /*
