@@ -337,7 +337,7 @@ generic_mozilla_int_notifier(GConfClient *client,
 		case GCONF_VALUE_INT:	mozilla_preference_set_int(user_data,
 						gconf_value_get_int(entry->value));
 					break;
-		case GCONF_VALUE_BOOL:	mozilla_preference_set_int(user_data,
+		case GCONF_VALUE_BOOL:	mozilla_preference_set_boolean(user_data,
 						gconf_value_get_bool(entry->value));
 					break;
 		case GCONF_VALUE_STRING:	mozilla_preference_set(user_data,
@@ -390,6 +390,15 @@ void mozilla_default_charset_notifier(GConfClient *client,
 	{
 		cur_charset = g_hash_table_lookup (charsets, 
 						   charset_title);
+		if (cur_charset != NULL) 
+		{
+			mozilla_preference_set ("intl.charset.default", cur_charset);
+		} 
+		else 
+		{
+			g_print ("warning: unknown default-charset, fallback to %s.\n", "ISO-8859-1");
+			mozilla_preference_set ("intl.charset.default", "ISO-8859-1");
+		}
 
 		mozilla_preference_set ("intl.charset.default", cur_charset);
 	}
@@ -440,13 +449,19 @@ mozilla_http_version_notifier(GConfClient *client,
 			      gpointer user_data)
 {
 	gchar *http_version = NULL;
+	int value;
 
-	switch (eel_gconf_get_integer (CONF_NETWORK_HTTP_VERSION))
+	value = eel_gconf_get_integer (CONF_NETWORK_HTTP_VERSION);
+	switch (value)
 	{
 		case 0: http_version = g_strdup ("1.0");
 			break;
 		case 1: http_version = g_strdup ("1.1");
 			break;
+		default:
+			g_warning ("mozilla_http_version_notifier: "
+				   "unsupported value: %d", value);
+			return;
 	}
 
 	mozilla_preference_set ("network.http.version", http_version);
@@ -549,6 +564,14 @@ mozilla_autodetect_charset_notifier(GConfClient *client,
 				    gpointer user_data)
 {
 	int charset = eel_gconf_get_integer (CONF_LANGUAGE_AUTODETECT_CHARSET);
+	if (charset < 0 || 
+	    charset >= (int)(sizeof(autodetect_charset_prefs)
+		             / sizeof(autodetect_charset_prefs[0])))
+	{
+		g_warning ("mozilla_autodetect_charset_notifier: "
+			   "unsupported value: %d", charset);
+		return;
+	}
 	mozilla_preference_set ("intl.charset.detector", 
 				autodetect_charset_prefs[charset]);
 }
@@ -563,6 +586,13 @@ mozilla_default_font_notifier(GConfClient *client,
 	int default_font;
 
 	default_font = eel_gconf_get_integer (CONF_RENDERING_DEFAULT_FONT);
+	if (default_font < 0 || 
+	    default_font >= (int)(sizeof(font_types) / sizeof(font_types[0])))
+	{
+		g_warning ("mozilla_default_font_notifier: "
+			   "unsupported value: %d", default_font);
+		return;
+	}
 	mozilla_preference_set ("font.default", font_types[default_font]);
 }
 
