@@ -209,7 +209,7 @@ mozilla_prefs_set_user_agent ()
 
 	if (!default_user_agent)
 	{
-		if (uname (&name) == 0)
+		if (uname (&name) >= 0)
 		{
 			system = g_strdup_printf ("%s %s",
 						  name.sysname, 
