@@ -347,7 +347,9 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 						    NETSCAPE_BOOKMARKS, NULL);
 			BookmarkTranslator *t = bookmarks_io_netscape_init ();
 			BookmarkItem *loaded = t->load_from_file (fname, NULL);
-			bookmarks_merge_trees (bookmarks_root, loaded);
+			if (loaded) {
+				bookmarks_merge_trees (bookmarks_root, loaded);
+			}
 			g_free (fname);
 		}
 	}
@@ -443,8 +445,7 @@ new_user_druid_finish_cb (GtkWidget *widget, gpointer dummy)
 	{
 		embed_create_in_window (NULL, NULL, NULL,
 					EMBED_CREATE_LOAD_DEFAULT_URL |
-					EMBED_CREATE_FORCE_JUMP |
-					EMBED_CREATE_RAISE_WINDOW);
+					EMBED_CREATE_FORCE_JUMP);
 	}
 }
 
@@ -466,8 +467,7 @@ new_user_druid_cancel_cb (GtkWidget *widget, gpointer dummy)
 	{
 		embed_create_in_window (NULL, NULL, NULL, 
 					EMBED_CREATE_LOAD_DEFAULT_URL |
-					EMBED_CREATE_FORCE_JUMP |
-					EMBED_CREATE_RAISE_WINDOW);
+					EMBED_CREATE_FORCE_JUMP);
 	}
 }
 
@@ -484,8 +484,7 @@ new_user_druid_delete_event_cb (GtkWidget *widget, GdkEventAny *event,
 	{
 		embed_create_in_window (NULL, NULL, NULL,
 					EMBED_CREATE_LOAD_DEFAULT_URL |
-					EMBED_CREATE_FORCE_JUMP |
-					EMBED_CREATE_RAISE_WINDOW);
+					EMBED_CREATE_FORCE_JUMP);
 	}
 	
 	/* still destroy the druid window */
