@@ -233,8 +233,7 @@ panel_applet_button_press_cb (GtkWidget *button, GdkEventButton *event,
 	{
 		embed_create_in_window (NULL, NULL, NULL,
 					EMBED_CREATE_LOAD_DEFAULT_URL |
-					EMBED_CREATE_FORCE_JUMP |
-					EMBED_CREATE_RAISE_WINDOW);
+					EMBED_CREATE_FORCE_JUMP);
 		return TRUE;
 	}
 
@@ -249,8 +248,7 @@ static void
 panel_applet_bookmark_activate_cb (AppletWidget *applet, BookmarkItem *bi)
 {
 	embed_create_in_window (NULL, NULL, bi->url,
-				EMBED_CREATE_FORCE_JUMP |
-				EMBED_CREATE_RAISE_WINDOW);
+				EMBED_CREATE_FORCE_JUMP);
 }
 
 /**
