@@ -354,6 +354,7 @@ char *proxy_sg [] =
 	"no_proxies_for_entry",
 	NULL,
 	"proxy_auto_url_entry",
+	"proxy_auto_reload_button",
 	NULL,
 	END_STRING
 };
@@ -518,6 +519,7 @@ prefs_lookup_widgets (GladeXML *gxml)
 
 		{ "lang_entry", &(pd->lang_entry) },
 		{ "proxy_auto_url_entry", &(pd->proxy_auto_url_entry) },
+		{ "proxy_auto_reload_button", &(pd->proxy_auto_reload_button) },
 
 		/* fonts */
 		{ "serif_font_fpicker",  &(pd->serif_font_picker) },
@@ -1085,7 +1087,9 @@ prefs_close (void)
 	/* This unselect is necessary because for some reason, gtk is
 	 * unselecting the row without sending the unselect signal when the
 	 * window is hidden.  -Daniel */
+	/* Do the same with the language list -Yanko */
 	gtk_clist_unselect_all (GTK_CLIST (pd->mime_list));
+	gtk_list_unselect_all (GTK_LIST (pd->lang_list));
 
 	pd->parent_window = NULL;
 
