@@ -23,6 +23,8 @@
 #include "prefs_utils.h"
 #include "mozilla.h"
 #include "dialog.h"
+#include "window.h"
+#include "embed.h"
 
 #include <libgnomeui/gnome-dialog-util.h>
 #include <stdlib.h>
@@ -63,6 +65,34 @@ typedef struct
 }
 PrintWidgets;
 
+typedef struct
+{
+	GtkWidget *dialog;
+	GtkWidget *pageSpinner;
+	GtkWidget *pageTotal;
+	gint numPages;
+
+	gboolean menuBarState;
+	gboolean toolBarState;
+	gboolean bookmarkBarState;
+	gboolean statusBarState;
+	gboolean notebookState;
+	gboolean dockState;
+
+	GaleonEmbed *embed;
+}
+PrintPreviewData;
+
+typedef enum
+{
+	PRINTPREVIEW_GOTO_PAGENUM = 0,
+	PRINTPREVIEW_PREV_PAGE = 1,
+	PRINTPREVIEW_NEXT_PAGE = 2,
+	PRINTPREVIEW_HOME = 3,
+	PRINTPREVIEW_END = 4
+} printPreviewNavType;
+
+
 static PrintInfo * print_get_info (PrintWidgets *pw);
 static void print_info_free (PrintInfo *info);
 static void print_options_set (PrintWidgets *widgets);
@@ -76,6 +106,12 @@ void print_preview_button_cb (GtkWidget *button, PrintWidgets *widgets);
 void print_cancel_button_cb (GtkWidget *button, PrintWidgets *widgets);
 void print_destroy_cb (GtkWidget *dialog, PrintWidgets *widgets);
 
+void print_preview_page_first_cb (GtkWidget *button, PrintPreviewData *ppd);
+void print_preview_page_last_cb (GtkWidget *button, PrintPreviewData *ppd);
+void print_preview_page_spinner_cb (GtkWidget *spinner, PrintPreviewData *ppd);
+void print_preview_close_cb (GtkWidget *button, PrintPreviewData *ppd);
+void print_preview_destroy_cb (GtkWidget *dialog, PrintPreviewData *ppd);
+
 /* returns a newly allocated PrintWidgets struct */
 static PrintWidgets * print_widgets_new()
 {
@@ -89,6 +125,8 @@ void print_show_dialog (GaleonEmbed *embed)
 	GladeXML *gxml;
 	PrintWidgets *pw;
 	GaleonWindow *window;
+	gboolean selected;
+	GSList *rb_group;
 
 	if (embed->print_dialog) return;
 
@@ -109,6 +147,18 @@ void print_show_dialog (GaleonEmbed *embed)
 	embed->print_dialog = pw->print_dialog;
 	pw->embed = embed;
 	print_options_get (pw);
+
+	selected = mozilla_can_cut_selection (embed) || 
+		   mozilla_can_copy_selection (embed);
+	rb_group = gtk_radio_button_group (GTK_RADIO_BUTTON(pw->all_pages_rb));
+	if (!selected && eel_gconf_get_integer (CONF_PRINT_ALL_PAGES) == 2) {
+		eel_gconf_set_integer (CONF_PRINT_ALL_PAGES, 0);
+		gtk_toggle_button_set_active 
+			(GTK_TOGGLE_BUTTON (g_slist_nth (rb_group, 2)->data),
+			 TRUE);
+	}
+	gtk_widget_set_sensitive (GTK_WIDGET (g_slist_nth (rb_group, 0)->data), 
+				  selected);
 	
 	/* show the print dialog */
 	gtk_widget_show (embed->print_dialog);
@@ -371,10 +421,22 @@ void
 print_preview_button_cb (GtkWidget *button, PrintWidgets *widgets)
 {
 	PrintInfo *info;
+//	GaleonEmbed *pp_embed;
+//	DisplayType type = DISPLAY_NORMAL;
 
 	info = print_get_info (widgets);
+/*
+	pp_embed = embed_create_after_embed (widgets->embed, TRUE,
+					     widgets->embed->location,
+					     EMBED_CREATE_FORCE_JUMP);
+	mozilla_copy_page (pp_embed, widgets->embed, type);
+	mozilla_print (pp_embed, info, TRUE);
+*/
 	mozilla_print (widgets->embed, info, TRUE);
+
 	print_info_free (info);
+
+	gtk_widget_destroy (widgets->print_dialog);
 }
 
 void
@@ -389,3 +451,122 @@ print_destroy_cb (GtkWidget *dialog, PrintWidgets *widgets)
 	widgets->embed->print_dialog = NULL;
 	g_free (widgets);
 }
+
+void print_show_preview_controls (GaleonEmbed *embed)
+{	
+	GladeXML *gxml;
+	PrintPreviewData *ppd;
+	GaleonWindow *window;
+	gchar *numPagesStr = 0;
+	GtkAdjustment *adj;
+
+	ppd = g_new0(PrintPreviewData, 1);
+
+	gxml = glade_widget_new ("print-preview.glade", "print_preview_panel",
+				 &ppd->dialog, ppd);
+	
+	window = embed->parent_window;
+	if (window)
+	{
+		dialog_set_parent (ppd->dialog, window->wmain);
+
+		ppd->menuBarState = GTK_CHECK_MENU_ITEM(window->view_menubar)->active;
+		ppd->toolBarState = GTK_CHECK_MENU_ITEM(window->view_toolbar)->active;
+		ppd->bookmarkBarState = GTK_CHECK_MENU_ITEM(window->view_bookmarks)->active;
+		ppd->statusBarState = GTK_CHECK_MENU_ITEM(window->view_statusbar)->active;
+
+		window_menubar_hide(window);
+		window_toolbar_hide(window);
+		window_statusbar_hide(window);
+		bookmarks_toolbars_set_visibility(window, FALSE);
+
+		if(window->notebook)
+		{
+			ppd->notebookState = GTK_NOTEBOOK(window->notebook)->show_tabs;
+			gtk_notebook_set_show_tabs(GTK_NOTEBOOK(window->notebook), FALSE);
+		}
+
+		if(window->docked)
+		{
+			ppd->dockState = GTK_WIDGET_VISIBLE(window->docked);
+			gtk_widget_hide(window->hpaned);
+			gtk_widget_hide(window->notebook);
+			gtk_widget_reparent(window->notebook, window->hbox);
+			gtk_widget_show(window->notebook);
+		}
+
+	}
+
+	ppd->numPages = mozilla_print_preview_num_pages (embed);
+
+	ppd->embed = embed;
+
+	ppd->pageSpinner = glade_xml_get_widget(gxml,
+						"print_preview_page_spinner");
+
+	ppd->pageTotal = glade_xml_get_widget(gxml,
+					      "print_preview_page_total");
+
+	adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ppd->pageSpinner));
+	adj->upper = ppd->numPages;
+	gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(ppd->pageSpinner), adj);
+
+	numPagesStr = g_strdup_printf("%d", ppd->numPages);
+	gtk_label_set_text (GTK_LABEL(ppd->pageTotal), numPagesStr);
+	g_free(numPagesStr);
+
+	/* show the print dialog */
+	gtk_widget_show (ppd->dialog);
+
+	gtk_object_unref (GTK_OBJECT (gxml));	
+}
+
+void print_preview_page_first_cb (GtkWidget *button, PrintPreviewData *ppd)
+{
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(ppd->pageSpinner), 1);
+}
+
+void print_preview_page_last_cb (GtkWidget *button, PrintPreviewData *ppd)
+{
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(ppd->pageSpinner),
+				  ppd->numPages);
+}
+
+void print_preview_page_spinner_cb (GtkWidget *spinner, PrintPreviewData *ppd)
+{
+	printPreviewNavType navType = PRINTPREVIEW_GOTO_PAGENUM;
+	mozilla_print_preview_navigate(ppd->embed, navType,
+			gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinner)));
+}
+
+void print_preview_close_cb (GtkWidget *button, PrintPreviewData *ppd)
+{
+	gtk_widget_destroy (ppd->dialog);
+}
+
+void print_preview_destroy_cb (GtkWidget *dialog, PrintPreviewData *ppd)
+{
+	GaleonWindow *window;
+
+	mozilla_print_preview_close(ppd->embed);
+
+	window = ppd->embed->parent_window;
+	if(window)
+	{
+		if(ppd->menuBarState) window_menubar_show(window);
+		if(ppd->toolBarState) window_toolbar_show(window);
+		if(ppd->statusBarState) window_statusbar_show(window);
+		bookmarks_toolbars_set_visibility(window, ppd->bookmarkBarState);
+		if(ppd->notebookState) gtk_notebook_set_show_tabs(GTK_NOTEBOOK(window->notebook), TRUE);
+		if(ppd->dockState)
+		{
+			gtk_widget_hide(window->notebook);
+			gtk_widget_reparent(window->notebook, window->hpaned);
+			gtk_widget_show(window->notebook);
+			gtk_widget_show(window->hpaned);
+		}
+	}
+
+	g_free (ppd);
+}
+
