@@ -16,7 +16,16 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+typedef enum
+{
+	TOOLBAR_TYPE_NONE,
+	TOOLBAR_TYPE_TOOLBAR,
+	TOOLBAR_TYPE_MENUBAR
+} ToolbarType;
+
 void toolbar_create (GaleonWindow *window, gboolean show);
 void toolbar_recreate (GaleonWindow *window);
+void toolbar_set_dock_style (GtkWidget *dock, ToolbarType type,
+		             GtkToolbarStyle toolbar_style);
 
 extern const ToolbarItem toolbar_items[TOOLBAR_ITEM_COUNT];
