@@ -16,13 +16,15 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "prefs.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 /* toolbar_editor.c */
-void toolbar_editor_init (PreferencesDialog *pd);
-void toolbar_editor_save (PreferencesDialog *pd);
+void toolbar_editor_init (void);
+void toolbar_editor_save (void);
 gint toolbar_editor_insert_item (GtkCList *clist, ToolbarItemType type);
 void toolbar_editor_remove_item (GtkCList *clist, ToolbarItemType type);
 
