@@ -179,33 +179,24 @@ void
 window_update_location_entry_text (GaleonWindow *window)
 {
 	GaleonEmbed *embed;
+	gchar *location;
 
 	embed = window->active_embed;
 	return_if_not_embed (embed);
 
 	/* if we have a saved modified location, it means that we just
-	 * switched back to this embed.  set the modified text. */
+	 * switched back to this embed.  use the modified text. */
 	if (embed->modified_location)
-	{
-		window_set_location_entry_text (window,
-						embed->modified_location);
-	}
-	/* if we don't have a saved location, but location_is_modified is
-	 * set, the user is currently modifying the location. */
-	else if (embed->location_is_modified)
-	{
-		/* do nothing */
-	}
-	/* otherwise, just set the location entry.  clear it if we're
-	 * currently at about:blank. */
-	else
-	{
-		if (embed->location && strcmp (embed->location, "about:blank"))
-			window_set_location_entry_text (window,
-							embed->location);
-		else
-			window_set_location_entry_text (window, NULL);
-	}
+		location = embed->modified_location;
+	/* otherwise, use the actual location */
+	else location = embed->location;
+
+	/* clear if we'd be setting to about:blank */
+	if (location && !strcmp (location, "about:blank"))
+		location = NULL;
+
+	/* finally, set the location entry text */
+	window_set_location_entry_text (window, location);
 }
 
 /**
@@ -1407,29 +1398,27 @@ window_close (GaleonWindow *window)
 	/* any children? */
 	if (window->embed_list != NULL)
 	{
-		gboolean already_dont_autosave = FALSE;
+		gboolean prev_dont_autosave;
 
-		/* check if dont autosave is already set */
-		if (dont_autosave_session)
-			already_dont_autosave = TRUE;
-		else dont_autosave_session = TRUE;
+		/* get the current state of the dont_autosave flag */
+		prev_dont_autosave = dont_autosave_session;
 
-		/* don't save session after every close */
+		/* enable it, so we don't save the session a bunch of times
+		 * as we close all the embeds */
 		dont_autosave_session = TRUE;
 
-		/* close any embed children */
+		/* close any embed children -- closing the last one will
+		 * call window_close again, which will execute the else
+		 * block below */
 		copy = g_list_copy (window->embed_list);
 		g_list_foreach (copy, (GFunc) embed_close, NULL);
 		g_list_free (copy);
 
-		/* save session now */
-		if (!already_dont_autosave)
-		{
-			dont_autosave_session = FALSE;
-			session_autosave ();
-		}
+		/* reset the dont_autosave flag to its previous state */
+		dont_autosave_session = prev_dont_autosave;
 
-		/* window_close will now be called again automatically */
+		/* save session now */
+		session_autosave ();
 	}
 	else
 	{
