@@ -615,7 +615,7 @@ xbel_read_node (BookmarkLoaderInfo *li,
 		gchar *title = get_child_value_string (node, "title");
 		if (!title) 
 		{
-			if (strcmp (node->name, "xbel"))
+			if (!strcmp (node->name, "xbel"))
 				title = g_strdup (_("XBEL bookmarks root"));
 			else
 				title = g_strdup (_("Untitled folder"));
